<?php
require __DIR__ . "/../vendor/autoload.php";
require "autoloader.php";
require __DIR__ . "/../vendor/zeedhi/framework/bootstrap.php";
require __DIR__ . "/../vendor/teknisa/libraries/scripts/bootstrap.php";

// Load specified services and parameters
$instanceManager = \Zeedhi\Framework\DependencyInjection\InstanceManager::getInstance();

$routes[] = __DIR__ . '/../vendor/teknisa/libraries/libRoutes.json';
$routes[] = __DIR__ . "/../vendor/zeedhi/preferences/routes.json";
$routes[] = __DIR__ . '/../routes.json';

$instanceManager->loadFromFile('../config/db.xml');
$instanceManager->loadFromFile('../config/environment.xml');
$instanceManager->loadFromFile('../config/general.xml');
$instanceManager->loadFromFile('../config/parameters.xml');
$instanceManager->loadFromFile('../vendor/zeedhi/devops_entities/environment.xml');
$instanceManager->loadFromFile('../vendor/zeedhi/preferences/services.xml');

$container = $instanceManager->getContainer();
if ($container->hasParameter('use_zhu_acl') && $container->getParameter('use_zhu_acl') == 'true'){
    $acl = __DIR__ . '/../../modules/acl/backend/config/acl.xml';

    if(realpath($acl)){
        $instanceManager->loadFromFile($acl);
    }
    require __DIR__ . "/../vendor/zeedhi/zhuaclbackend/service/bootstrap.php";
    $routes[] = __DIR__ . "/../vendor/zeedhi/zhuaclbackend/config/routes.json";
}

if ($container->hasParameter('use_zhu_update') && $container->getParameter('use_zhu_update') == 'true'){
    $zhu = __DIR__ . '/../../modules/acl/backend/config/zhuUpdate.xml';
    $zhu2 = __DIR__ . '/../vendor/zeedhi/zhuentities/services.xml';

    if(realpath($zhu) && realpath($zhu2) ){
        $instanceManager->loadFromFile($zhu);
        $instanceManager->loadFromFile($zhu2);
    }
}

if ($container->hasParameter('use_zhu_log') && $container->getParameter('use_zhu_log') == 'true') {
    $zhuDateFormat = $container->getParameter('zhuLogDateFormat');
    $ignoredRoutes = $container->hasParameter('logIgnoredRoutes') ? $container->getParameter('logIgnoredRoutes') : [];

    require __DIR__ . "/../vendor/zeedhi/zhulog/config/bootstrap.php";
    $routes[] = __DIR__."/../vendor/zeedhi/zhulog/config/routes.json";

    $log = __DIR__.'/../../modules/acl/backend/config/log.xml';
    if(realpath($log)){
        $instanceManager->loadFromFile($log);
    }

    $libLog = __DIR__.'/../vendor/teknisa/libraries/zhuLog.xml';
    if(realpath($libLog)){
        $instanceManager->loadFromFile($libLog);
    }

    $container->setParameter('logIgnoredRoutes', $ignoredRoutes);
    $container->setParameter('zhuLogDateFormat', $zhuDateFormat);
}

$instanceManager->loadFromFile('../services.xml');
$instanceManager->registerService('routeParser', new \Zeedhi\Framework\Routing\Parsers\JSONFormat($routes));
$instanceManager->compile();

use Doctrine\DBAL\Types\Type;

Type::overrideType(Type::DATE, 'Setforms2\Shared\ORM\DateType');
Type::overrideType(Type::DATETIME, 'Setforms2\Shared\ORM\DateTimeType');

$entity_manager = $instanceManager->getService('entityManager');
\Setforms2\Shared\Registry::store("EntityManager", $entity_manager);
\Setforms2\Shared\Registry::store("EntityManagerConnection", $entity_manager->getConnection());
