<?php
namespace Controller;

use Exception\CustomException;
use Util\InstanceProvider;
use Util\Utilities;
use Zeedhi\Framework\Controller\Simple;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Util\Messages;

class Login extends Simple
{

    /** @var \Service\Login $loginService */
    protected $loginService;
    /** @var Utilities $environment */
    protected $environment;

    public function __construct(\Service\Login $loginService, Utilities $environment) {
        $this->loginService = $loginService;
        $this->environment = $environment;
    }

    public function login(Request\Filter $request, Response $response) {
        try {
            $conditions = $request->getFilterCriteria()->getConditions();
            $cdcliente   = Utilities::getValueInFilterDataSourceByName($conditions,'CDCLIENTE');
            $userId   = Utilities::getValueInFilterDataSourceByName($conditions,'USER_ID');
            $resultSet  = $this->loginService->login($userId,$cdcliente);
            $response->addDataSet(new DataSet('login', array($resultSet)));
        } catch (CustomException $e) {
            $response->setError(Utilities::getErrorObjectFromException($e));
        }
    }

    public function logout(Request\Filter $request, Response $response) {
        $this->loginService->clearSession();
        $this->clearSession();
    }

    public function clearSession() {
        $this->loginService->destroySession();
    }

    public function updateCustomerInSession(Request\Filter $request, Response $response) {
        try {
            $conditions = $request->getFilterCriteria()->getConditions();
            $customerCode = Utilities::getValueInFilterDataSourceByName($conditions, 'CDCLIENTE');
            InstanceProvider::getSession()->set('CDCLIENTE', $customerCode);
            $response->addDataSet(new DataSet('data', array("IS_TEKNISA_CLIENT" => Utilities::currentClienteIsTeknisa($customerCode))));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(),$e->getCode()));
        }
    }
}