<?php
namespace Service;

use Doctrine\ORM\EntityManager;
use Exception\CustomException;
use Service\Integration\CRMIntegration;
use Service\Integration\SMSoliservIntegration;
use Teknisa\Model\Crm\Entities\Contato;
use Teknisa\Model\Geral\Entities\Operador;
use Teknisa\Model\Teknisa\Entities;
use Util\Utilities;
use Util\Messages;
use Zeedhi\DevOps\Model\Projectmanagement\Entities\Task;
use Util\InstanceProvider;
use Zeedhi\DevOps\Model\Projectmanagement\Entities\Comments;
use Zeedhi\DevOps\Model\Projectmanagement\Entities\TaskStatus;
use Zeedhi\DevOps\Model\Projectmanagement\Entities\TaskType;
use Zeedhi\DevOps\Util\Utilities as DevOpsUtilities;
use Util\Mail\Core\SmtpMail;
use Util\Mail\ProductDeployMail;
use Zend\Mime\Part as ZendMimePart;
use Zend\Mime\Message as ZendMimeMessage;
use Zeedhi\Framework\Remote\FileUploader\Server as ServerFileUploader;

class Call
{

    const TYPE_ISSUE        = Entities\CpCall::TYPE_ISSUE;
    const TYPE_CASO         = Entities\CpCall::TYPE_CASO;
    const TYPE_RICH_CLIENT  = Entities\CpCall::TYPE_RICH_CLIENT;
    const TYPE_TICKET  = Entities\CpCall::TYPE_TICKET;

    const STAGE_ESPEC  = '050';
    const STAGE_RETUN  = '499';
    const STAGE_IMFOR  = '057';

    const MAIN_TYPE_OF_PRODUCT_CALL = "P";
    const MAIN_TYPE_OF_SUPPORT_CALL = "S";
    const CRM_CLOSE = 4;
    const CRM_INIC = 2;

    const FIRST_COMMENTS = 'Abertura de chamado.';

    /** @var EntityManager $entityManager */
    protected $entityManager;
    /** @var \Util\Utilities $environment */
    protected $environment;
    /** @var ServerFileUploader $zhFileUploader */
    protected $zhFileUploader;
    /** @var \Teknisa\Model\Teknisa\Repositories\CpCall $callRepository */
    public $callRepository;
    /** @var \Teknisa\Model\Geral\Repositories\Paramgeral $paramGeralRepository */
    protected $paramGeralRepository;
    /** @var \Teknisa\Model\Geral\Repositories\Operador $operadorRepository */
    protected $operadorRepository;
    /** @var \Service\Integration\INTRANETIntegration $intranetIntegrationService */
    protected $intranetIntegrationService;

    protected $orderReverse = false;
    protected $orderBy = 'ID';

    private $smtpMail;
    private $baseDir = '';

    public function __construct(EntityManager $entityManager, \Util\Utilities $environment,
                                SmtpMail $smtpMail, \Factory\LogFaseTram $logFaseTramFactory, \Factory\CpCustomerPreferences $cpCustomerPreferencesFactory, ServerFileUploader $zhFileUploader) {
        $this->entityManager = $entityManager;
        $this->environment = $environment;
        $this->zhFileUploader = $zhFileUploader;
        $this->operadorRepository = $this->entityManager->getRepository('\Teknisa\Model\Geral\Entities\Operador');
        $this->callRepository = $this->entityManager->getRepository('\Teknisa\Model\Teknisa\Entities\CpCall');
        $this->paramGeralRepository = $this->entityManager->getRepository('\Teknisa\Model\Geral\Entities\Paramgeral');
        $this->logfaseTramRepository    = $this->entityManager->getRepository('\Teknisa\Model\Teknisa\Entities\Logfasetram');

        $this->smtpMail = $smtpMail;
        $this->baseDir = __DIR__.DIRECTORY_SEPARATOR.'..'.DIRECTORY_SEPARATOR.'Util'.DIRECTORY_SEPARATOR.'EmailTemplates'.DIRECTORY_SEPARATOR;

        $this->logFaseTramFactory = $logFaseTramFactory;
        $this->cpCustomerPreferencesFactory = $cpCustomerPreferencesFactory;
    }


    private function encodeAndSend($to, $subject, $message)
    {
        $message        = utf8_decode($message);
        $mimePart       = new ZendMimePart($message);
        $mimeMessage    = new ZendMimeMessage();
        $mimePart->type = 'text/html';
        $mimeMessage->setParts(array($mimePart));
        $this->sendHtml($to, $subject, $mimeMessage);
    }

    private function getEmailMessage($email){
        $message = $this->fileGetContentUtf8($this->baseDir . 'mail.html');
        $message = str_replace("%title%",$email["DSREFESOLI"],$message);
        $message = str_replace("%name%",$email["NMOPERADOR"],$message);
        $message = str_replace("%left%",$email["SOLI"],$message);
        $message = str_replace("%comment%",
            empty($email["DSALFASETRAM"]) ? "" :
            "<tr>
                <td class='title'>
                    <div>Comentário</div>
                </td>
            </tr>
            <tr>
                <td class='description'>
                    <div class='divTable'>
                        <div >".$email["DSALFASETRAM"]."</div>
                    </div>
                </td>
            </tr>"
            ,$message);
        $message = str_replace("%date%",$email["DTALFASETRAM"],$message);
        $message = str_replace("%action%",$email["TXACAOLFTRAM"],$message);
        return $message;
    }

    public function sendMail()
    {
        try{
            $cpCallRepository = InstanceProvider::getRepository()->getCpCall();
            $emails = $cpCallRepository->getUnsentEmails();
            foreach($emails as $i=>$email){
                $message = $this->getEmailMessage($email);
                try {
                    $this->encodeAndSend($email["DSEMAILOPER"], "Solicitação ".$email["DSREFESOLI"], $message);
                    $success = "\033[1;32mSucesso";
                    $this->updateEmailEnvProductCallComment($email["CDFILIAL"],$email["NRSOLISERV"], new \DateTime(str_replace("/","-",$email["DTALFASETRAM"])));
                }catch(\Exception $e){
                    $success = "\033[1;31mFalha\n\033[0mMotivo da falha: \033[1;31m".$e->getMessage()."\033[0m";
                }
                $this->showEmailLog($email,$success);
            }
        }catch (\Exception $e){
            throw $e;
        }
    }

    private function showEmailLog($email,$success){
        $now = new \DateTime();
        echo
"Solicitante: ".$email["SOLI"]." (".$email["SOLICD"].")
Responsável: ".$email["RESP"]." (".$email["RESPCD"].")
Data: ".$now->format('d/m/Y h:i')."
Ação: ".$email["TXACAOLFTRAM"]."
Número da Solicitação: ".$email["NRSOLISERV"]."
Referência: ".$email["DSREFESOLI"]."
Status do envio de email: $success\033[0m
\033[1;34m==================================================================\033[0m
";
    }

    private function sendHtml($to, $subject, $message)
    {
        $this->smtpMail->setFrom("pm@teknisa.com");
        $this->smtpMail->setTo($to);
        $this->smtpMail->setSubject($subject);
        $this->smtpMail->setBody($message);
        $this->smtpMail->sendHtml();
    }

    private function fileGetContentUtf8($fileName)
    {
        $content = file_get_contents($fileName);
        return mb_convert_encoding($content, 'UTF-8', mb_detect_encoding($content, 'UTF-8, ISO-8859-1', true));
    }

    private function updateEmailEnvProductCallComment($cdfilial, $nrsoliserv, $dtalfasetram){
        $connection = $this->entityManager->getConnection();
        try {
            $connection->beginTransaction();
            $log = $this->logfaseTramRepository->findOneBy(array('cdfilial' => $cdfilial,'nrsoliserv' => $nrsoliserv,'dtalfasetram' => $dtalfasetram));
            $log = $this->logFaseTramFactory->updateEmailEnv($log);
            $this->entityManager->persist($log);
            $this->entityManager->flush();
            $connection->commit();

        } catch(\Exception $e) {
            $connection->rollBack();
            throw $e;
        }

    }

    public function findMenuItems($pmProductId)
    {
        $omIntegration = InstanceProvider::getService()->getOmIntegration();
        return $omIntegration->findMenuItems($pmProductId);
    }

    public function getLinkDownloadBudgetSm($issueId)
    {
        $linkDownloadBudgetSm =  \Util\Instance\Service::getPmIntegration()
            ->getLinkDownloadBudgetSm($issueId);
        \Util\Instance\Service::getAttachment()->downloadBudgetSm($linkDownloadBudgetSm);

        $this->zhFileUploader->deleteFile($linkDownloadBudgetSm);
    }
    public function getLinkDownloadBudgetSmSoliserv($nrsoliserv)
    {
        $nrorg = Utilities::getEnvironment()->getCurrentOrganizationId();
        $linkDownloadBudgetSm =  Integration\SMSoliservIntegration::linkDownloadBudgetSm($nrsoliserv,$nrorg);
        \Util\Instance\Service::getAttachment()->downloadBudgetSm($linkDownloadBudgetSm);

        $this->zhFileUploader->deleteFile($linkDownloadBudgetSm);
    }

    public function findCallBudget($nrsoliserv,$nameProposal,$nameDiscount,$lang)
    {
        $cpCallRepository = InstanceProvider::getRepository()->getCpCall();
        return $cpCallRepository->findCallBudget($nrsoliserv,$nameProposal,$nameDiscount,$lang,$this->environment->currentUserIsTeknisa());
    }

    public function getPageRequisite($nrsoliserv)
    {
        InstanceProvider::getService()->getAttachment()->getPageRequisite($nrsoliserv);
    }

    public function findProductVersion($pmProductId,$pageSize,$page,$filter)
    {
        $omIntegration = InstanceProvider::getService()->getDevOpsIntegration();
        return $omIntegration->findProductVersion($pmProductId,$pageSize,$page,$filter);
    }

    public function findCallTypeIssue($id)
    {
        $cpCallRepository = InstanceProvider::getRepository()->getCpCall();
        $situationFilters = $this->getAllSituationDividedByType();
        return $cpCallRepository->findCallTypeIssue($id,$situationFilters);
    }

    public function findCallTypeRichClient($id)
    {
        $cpCallRepository = InstanceProvider::getRepository()->getCpCall();
        $userId = $this->environment->getCurrentUserId();
        $situationFilters = $this->getAllSituationDividedByType();
        return $cpCallRepository->findCallTypeRichClient($id,$userId,$situationFilters);
    }

    public function findSoliservById($soliservId)
    {
        $situationFilters = $this->getAllSituationDividedByType();
        $userId = $this->environment->getCurrentUserId();
        return InstanceProvider::getRepository()->getCpCall()->findSoliservById($soliservId, $situationFilters, $userId);
    }

    public function getBeginAndEndCdfasetram()
    {
        $cpCallRepository = InstanceProvider::getRepository()->getCpCall();
        $paramgeral = $cpCallRepository->getBeginAndEndCdfasetram($this->environment->getCurrentOrganizationId());
        return array($paramgeral['CDFATRAMIWEB'],$paramgeral['CDFATRAMENCE']);
    }

    public function getAllSituationDividedByType($eliasSoliserv = 'SER')
    {
        $cpCallRepository = InstanceProvider::getRepository()->getCpCall();
        $waitingTicket = Utilities::getWaitingTicket();
        $allSituation  = $cpCallRepository->getAllSituation($this->environment->getCurrentOrganizationId(),$waitingTicket);
        $processingType = Utilities::getExcludedWaitingCustomerLogFaseTram();
        $cdcliente = Utilities::getEnvironment()->getCurrentCdCliente();
        $waitingGEP = " (EXISTS (SELECT CDOPERADOR FROM OPERADOR WHERE CDCLIENTE = '$cdcliente' AND CDOPERADOR = $eliasSoliserv.CDOPERRESP AND $eliasSoliserv.CDFASETRAM NOT IN ('$processingType')" ;
        list($openedGep, $closedGep) = $this->getBeginAndEndCdfasetram();
        $allSituationDividedByType = array(
            'GEP' => [
                'CLOSED' => " $eliasSoliserv.CDFASETRAM = $closedGep OR ($eliasSoliserv.IDSOLIENCE = 'S')",
                'OPENED' => " $eliasSoliserv.CDFASETRAM = $openedGep AND ($eliasSoliserv.IDSOLIENCE <> 'S')",
                'WAITING' => $waitingGEP." AND $eliasSoliserv.CDFASETRAM NOT IN ($closedGep) AND ($eliasSoliserv.IDSOLIENCE <> 'S'))) ",
                'IN_PROGRESS' => " $eliasSoliserv.CDFASETRAM NOT IN ($openedGep,$closedGep) AND ($eliasSoliserv.IDSOLIENCE <> 'S') ",
            ],
            'PM' => [
                'CLOSED' => [],
                'OPENED' => [],
                'WAITING' => [],
                'IN_PROGRESS' => [],
            ],
            'CRM' => [
                'CLOSED' => [],
                'OPENED' => [],
                'WAITING' => [],
                'IN_PROGRESS' => [],
            ],
        );
        foreach ($allSituation as $situation) {
            $allSituationDividedByType[$situation['PRODUCT']][$situation['SITUATION']][] = $situation['ID'];
        }
        return $allSituationDividedByType;
    }

    public function situationFilter($situationTypes, $eliasContato = 'CONT', $eliasTaskStattus = 'TS', $eliasSoliserv = 'SER')
    {
        $situationFilter = array(
            'ISSUE' =>'',
            'CONTATO' =>'',
            'SOLISERV' =>'',
        );
        if(!empty($situationTypes)){
            $issue = array();
            $contato = array();
            $soliserv  = array();
            asort($situationTypes);
            $isWaitingIssue= '';
            $allSituationDividedByType = $this->getAllSituationDividedByType($eliasSoliserv);
            foreach ($situationTypes as $situationType) {
                if ($situationType == Entities\CpCall::CLOSE_SITUATION) {
                    $issue = $allSituationDividedByType['PM']['CLOSED'];
                    $contato = $allSituationDividedByType['CRM']['CLOSED'];
                    $soliserv[] = $allSituationDividedByType['GEP']['CLOSED'];
                } else if ($situationType == Entities\CpCall::OPEN_SITUATION) {
                    $issue = array_merge($issue,$allSituationDividedByType['PM']['OPENED']);
                    $contato = array_merge($contato,$allSituationDividedByType['CRM']['OPENED']);
                    $soliserv[] = $allSituationDividedByType['GEP']['OPENED'];
                } else  if ($situationType == Entities\CpCall::PROGRESS_SITUATION) {
                    $issue = array_merge($issue,$allSituationDividedByType['PM']['IN_PROGRESS']);
                    $contato = array_merge($contato,$allSituationDividedByType['CRM']['IN_PROGRESS']);
                    $soliserv[] = $allSituationDividedByType['GEP']['IN_PROGRESS'];
                } else if ($situationType == Entities\CpCall::WAITING_SPONSOR_SITUATION) {
                    $issue = array_merge($issue,$allSituationDividedByType['PM']['WAITING']);
                    $contato = array_merge($contato,$allSituationDividedByType['CRM']['WAITING']);
                    $soliserv[] = $allSituationDividedByType['GEP']['WAITING'];
                    $isWaitingIssue = true;
                }
            }
            $question = <<<SQL
                         'Q' = (SELECT NVL(CONTROL,'I') FROM (SELECT CTY.CONTROL
                                                FROM COMMENTS COM
                                                LEFT JOIN COMMENT_TYPE CTY ON(COM.COMMENT_TYPE_ID = CTY.ID )
                                                WHERE COM.TASK = TAS.ID
                                                ORDER BY COM.ID DESC)
                                                WHERE ROWNUM = 1 )
SQL;

            if(!empty($issue)){
                $contatoString = implode($contato, ',');
                $issueString = implode($issue, ',');
                $soliservString = implode($soliserv, 'OR');
                $situationFilter['CONTATO'] = " AND $eliasContato.NRSEQSITUACAO IN($contatoString)";
                $situationFilter['SOLISERV'] = " AND ($soliservString)";

                if($isWaitingIssue){
                    $situationFilter['ISSUE'] = " AND ($eliasTaskStattus.ID IN($issueString) OR $question )";
                }else{
                    $situationFilter['SOLISERV'] .= "AND NOT ".$allSituationDividedByType['GEP']['WAITING'];
                    $situationFilter['ISSUE'] = " AND $eliasTaskStattus.ID IN($issueString)"." AND NOT ".$question;
                }
            }
        }
        return $situationFilter;
    }

    public function getCallTypeDescription($type) {
        switch($type) {
            case self::TYPE_ISSUE:
                $desc = Entities\CpCall::TYPE_ISSUE_DESC;
                break;
            case self::TYPE_RICH_CLIENT:
                $desc = Entities\CpCall::TYPE_RICH_CLIENT_DESC;
                break;
            case self::TYPE_CASO:
                $desc = Entities\CpCall::TYPE_CASO_DESC;
                break;
            case self::TYPE_TICKET:
                $desc = Entities\CpCall::TYPE_TICKET_DESC;
                break;
            default:
                $desc = '';
        }
        return $desc;
    }

    public function countCallsByType()
    {
        $cpCallRepository = InstanceProvider::getRepository()->getCpCall();
        $cdCliente = $this->environment->getCurrentCdCliente();
        $userId = $this->environment->getCurrentUserId();
        $filter = $this->situationFilter(array('O','P','W'));
        $counts = $cpCallRepository->countCallsByType($cdCliente, $userId, $filter);
        return $counts;
    }

    public function countCallsByLabel(){
        $cpCallRepository = InstanceProvider::getRepository()->getCpCall();
        $cdCliente = $this->environment->getCurrentCdCliente();
        $userId = $this->environment->getCurrentUserId();
        $filter = $this->situationFilter(array('O','P','W'));
        $result = $cpCallRepository->countCallsByLabel($cdCliente,$userId,$filter);
        return \Zeedhi\DevOps\Util\Utilities::fixBugChartPie($result,'QUANTITY','LABEL');
    }

    public function getLastQuestion($taskId){
        $cpCallRepository = InstanceProvider::getRepository()->getCpCall();
        return $cpCallRepository->getLastQuestion($taskId);
    }

    public function getCallGraphicForFourMonth()
    {
        $cpCallRepository = InstanceProvider::getRepository()->getCpCall();
        $cdCliente        = $this->environment->getCurrentCdCliente();
        $userId           = $this->environment->getCurrentUserId();
        $situationFilters = $this->getAllSituationDividedByType();
        $counts           = $cpCallRepository->getCallGraphicForFourMonth($cdCliente,$situationFilters,$userId);

        foreach ($counts as &$count) {
            $count['CLOSED'] = intval($count['CLOSED']);
            $count['OPENED'] = intval($count['OPENED']);
            $count['INPROGRESS'] = intval($count['INPROGRESS']);
            $count['WAITING'] = intval($count['WAITING']);
        }

        return $counts;
    }

    public function saveTicketCall($reference, $cdproduct, $description, $callRelationship,
                               Operador $reportedBy, $classification,$idSituacao = null, $cdopercont = null,
                               $reopen = false){

        $reporterEmail = $reportedBy->getDsemailoper();
        $callRelationship = $this->saveTicketInCRM($reference, $cdproduct, $description,
            $callRelationship, $reporterEmail,$classification, $idSituacao, $cdopercont, $reopen);
        /** @var Entities\CpCall $productCall */
        $productCall = InstanceProvider::getRepository()->getCpCall()->findOneBy(array("callRelationship" => $callRelationship));
        if (!$productCall) {
            $productCall = InstanceProvider::getFactory()->getCall()->createProductCall($reference, $reportedBy->getCdoperador(), $description, self::TYPE_TICKET,
                $callRelationship, $cdproduct);

            $this->entityManager->persist($productCall);
            $this->entityManager->flush();

        }
        return $productCall;
    }

    public function saveTicketInCRM($reference, $cdproduct, $description, $callRelationship,
                                    $reporterEmail,$cdmoticont,$idSituacao = null,$cdopercont = null,$reopen = false)
    {
        /** @var \Teknisa\Model\Comissaria\Entities\Prodarvoreprod $productEntity */
        $productEntity = InstanceProvider::getRepository()->getProdarvoreprod()
            ->findOneBy(array('cdproduto' => $cdproduct));
        list($classificationCrm, $cdnatucont, $nrfilaatendimento) = InstanceProvider::getRepository()->getCpCall()->getClassificationCrm($cdproduct);
        $nrarvoreprod = $productEntity->getNrarvoreprod();
        $nrtparvprod = $productEntity->getNrtparvprod();
        $dsarvoreprod = InstanceProvider::getRepository()->getCpCall()->getDsarvoreprod($cdnatucont, $classificationCrm, $nrtparvprod, $cdproduct);
        $nrparcinterlocutor = $this->environment->getCurrentUser()->getNrparcnegocio();
        $arrayContato = InstanceProvider::getFactory()->getContato()->createArrayTicket($reporterEmail, $classificationCrm,
            $nrtparvprod, $reference, $nrparcinterlocutor, $cdproduct, $description, $cdnatucont, $nrarvoreprod,
            $dsarvoreprod, $cdmoticont);
        $arrayContato['NRSEQCONTATO'] = $callRelationship;
        $arrayContato['NRSEQSITUACAO'] = $idSituacao;
        $arrayContato['CDOPERCONT'] = $cdopercont;
        $arrayContato['REOPEN'] = $reopen;
        $arrayContato['__is_new'] = !$callRelationship;
        $returnCRM = CRMIntegration::addTicketFromCRM(array($arrayContato));
        $callRelationship = $returnCRM['DADOS']['NRSEQCONTATO'];
        return intval($callRelationship);
    }

    public function createIssueInPM($reference, $product, $classification, $board, $description,
                                    $reportedBy, $callRelationship = null, $cdCliente = null)
    {
        $issue = null;
        $clienteRepository = InstanceProvider::getRepository()->getCliente();
        $pmIntegrationService = InstanceProvider::getService()->getPmIntegration();

        /** @var \Teknisa\Model\Geral\Entities\Operador $reporter */
        $reporter = InstanceProvider::getRepository()->getOperador()->findOneBy(array('cdoperador' => $reportedBy));

        $reporterName = $reporter->getNmoperador();

        if (!$cdCliente) {
            $cdCliente = $this->environment->getCurrentCdCliente();
            if(!$cdCliente){
                $cdCliente = $reporter->getCdcliente();
                $this->environment->setCdClienteCurrent($cdCliente);
            }
        }
        $client = $clienteRepository->findOneBy(array(
            'cdcliente' => $cdCliente,
            'nrorg' => $reporter->getNrorg()
        ));
        $reporterFrom = empty($client) ? null : $client->getNmfantclie();
        $sponsor = $pmIntegrationService->findResourceByExternalCode($cdCliente);
        $clienteResource = $pmIntegrationService->findResourceByExternalCode($reporter->getCdcliente());

        if (empty($sponsor)) {
            throw new CustomException('ERR_NOT_POSSIBLE_TO_CREATE_ISSUE_BECAUSE_CLIENT');
        }
        if (empty($clienteResource)) {
            /** @var \Teknisa\Model\Geral\Entities\Cliente $cliente */
            $cliente = $clienteRepository->findOneBy(array('cdcliente' => $cdCliente));
            $name = $cliente->getNmfantclie();
            throw new CustomException('ERR_CREATE_OR_UPDATE_PRODUCT_CALL_NOT_CUSTOMER_PM', 500, array('customerName' => $name));
        }

        if (!$callRelationship) {
            $issue = $pmIntegrationService->createIssueByProductCall($reference, $product, $classification,
                $description, $board, $reporterName, $reporterFrom, $sponsor->getId(), $reportedBy);
            if (empty($issue)) {
                throw new CustomException('ERR_NOT_POSSIBLE_TO_CREATE_ISSUE_BASED_ON_PRODUCT_CALL');
            }

            $callRelationship = $issue['ID'];
        }
        return $callRelationship;
    }

    public function setVersionIssue($callRelationship, $menuItemId = null, $menuVersionId = null)
    {
        /** @var Task $issue */
        $issue = InstanceProvider::getRepository()->getTask()->find($callRelationship);
        if ($menuVersionId) {
            $productVersion = InstanceProvider::getRepository()->getProductVersion()->findOneBy(array('id' => $menuVersionId));
            $issue->setProductVersion($productVersion);
        }
        $issue->setMenuItem($menuItemId);
        $this->entityManager->persist($issue);
        return $issue;
    }

    public function setFirstCommentIssue($product, $reportedBy, $callLabels = array(), $menuItemId, $menuVersionId,
                                        $issue, $productCall)
    {
        /** @var Comments $fistCommnet */
        $firstCommentDescription = InstanceProvider::getRepository()->getComments()
            ->getFistCommentsWithMessageVersion($issue->getId(), Messages::ISSUE_OPEN_VERSION, $product);
        if ($firstCommentDescription && ($menuVersionId || $menuItemId)) {
            $fistComment = InstanceProvider::getRepository()->getComments()->find($firstCommentDescription['ID']);
            $fistComment->setDescription($firstCommentDescription['DESCRIPTION']);
            $this->entityManager->persist($fistComment);
        }
        InstanceProvider::getService()->getCallLabels()->createCallRelationWithLabels($productCall->getNrcall(), $callLabels, $reportedBy);
    }

    public function saveSolicitacao($reference, $product, $productType, $productVersion, $useLicense, $classification,
                                    $description, $reportedBy, $productRelease, $productPatch,$cdCliente, $callRelationship = null)
    {
        $operadorRepository = InstanceProvider::getRepository()->getOperador();
        $reporter = $operadorRepository->findOneBy(array('cdoperador' => $reportedBy));
        if (!$reporter) {
            $reporter = $operadorRepository->findOneBy(array('cdoperador' => '1'));
        }

        if (!$callRelationship) {
            $P_CDCLIENTE = $cdCliente;
            $P_CDOPERADOR = $reporter->getCdoperador();
            $P_DSREFESOLI = $reference;
            $P_CDASSUNTO = $classification;
            $P_CDGRUPSIST = $productType;
            $P_CDSISTEMA = $product;
            $P_NRLICENCAUSO = $useLicense;
            $P_DSSOLISERV = $description;
            $P_DSVERSAO = $productVersion;
            $P_DSRELEASE = $productRelease;
            $P_DSPATCH = $productPatch;

            $callRelationship = InstanceProvider::getService()->getIntranetIntegration()->createAndPersistSolicitacao($P_CDCLIENTE, $P_CDOPERADOR,
                $P_DSREFESOLI, $P_CDASSUNTO, $P_CDGRUPSIST, $P_CDSISTEMA, $P_NRLICENCAUSO, $P_DSSOLISERV, $P_DSVERSAO,
                $P_DSRELEASE, $P_DSPATCH);

        }
        return $callRelationship;
    }

    public function getQueryAllCall($conditions, $mainType, $situationFilters, $clientePmFilter, $clienteIntranetFilter,
                                    $clienteCRMFilter)
    {
        list($callType, $whereFiltersProductCalls, $situationProductCalls, $specialFiltersProductCalls) = $this->buildWhereFilterToProductCalls($conditions);


        $queryPmCalls = $queryIntranetCalls = $queryCrmCalls = "";

        $cpCallRepository = InstanceProvider::getRepository()->getCpCall();
        if ($mainType == self::MAIN_TYPE_OF_PRODUCT_CALL) {
            if ($callType == Call::TYPE_ISSUE) {
                $queryPmCalls = $cpCallRepository->getQueryPmCalls($situationFilters, $specialFiltersProductCalls, $situationProductCalls['ISSUE'], $whereFiltersProductCalls, $clientePmFilter);
            } else if ($callType == \Service\Call::TYPE_RICH_CLIENT) {
                $queryIntranetCalls = $cpCallRepository->getQueryIntranetCalls($situationFilters, $specialFiltersProductCalls, $situationProductCalls['SOLISERV'], $whereFiltersProductCalls, $clienteIntranetFilter);
            } else if ($callType == \Service\Call::TYPE_TICKET) {

            } else {
                $queryPmCalls = $cpCallRepository->getQueryPmCalls($situationFilters, $specialFiltersProductCalls, $situationProductCalls['ISSUE'], $whereFiltersProductCalls, $clientePmFilter);
                $queryIntranetCalls = $cpCallRepository->getQueryIntranetCalls($situationFilters, $specialFiltersProductCalls, $situationProductCalls['SOLISERV'], $whereFiltersProductCalls, $clienteIntranetFilter);
            }
        } else if ($mainType == self::MAIN_TYPE_OF_SUPPORT_CALL) {
            if ($callType == \Service\Call::TYPE_TICKET) {
                $queryCrmCalls = $cpCallRepository->getQueryTicketCalls($situationFilters, $specialFiltersProductCalls, $situationProductCalls['CONTATO'], $whereFiltersProductCalls, $clienteCRMFilter);
            } else if ($callType == \Service\Call::TYPE_RICH_CLIENT || $callType == Call::TYPE_ISSUE) {

            } else {
                $queryCrmCalls = $cpCallRepository->getQueryTicketCalls($situationFilters, $specialFiltersProductCalls, $situationProductCalls['CONTATO'], $whereFiltersProductCalls, $clienteCRMFilter);
            }
        } else {
            $queryPmCalls = $cpCallRepository->getQueryPmCalls($situationFilters, $specialFiltersProductCalls, $situationProductCalls['ISSUE'], $whereFiltersProductCalls, $clientePmFilter);
            $queryIntranetCalls = $cpCallRepository->getQueryIntranetCalls($situationFilters, $specialFiltersProductCalls, $situationProductCalls['SOLISERV'], $whereFiltersProductCalls, $clienteIntranetFilter);
            $queryCrmCalls = $cpCallRepository->getQueryTicketCalls($situationFilters, $specialFiltersProductCalls, $situationProductCalls['CONTATO'], $whereFiltersProductCalls, $clienteCRMFilter);
        }

        $customQuery = $queryPmCalls;



        if ($queryIntranetCalls) {
            $customQuery .= (empty($customQuery) ? "" : " UNION ") . $queryIntranetCalls;
        }

        if ($queryCrmCalls) {
            $customQuery .= (empty($customQuery) ? "" : " UNION ") . $queryCrmCalls;
        }
        return $customQuery;
    }

    private function buildWhereFilterToSupportCalls($conditions)
    {
        $specialFilters = '';
        $situation = array(
            'ISSUE' =>'',
            'CONTATO' =>'',
            'SOLISERV' =>''
        );
        foreach($conditions as $key => $filter) {
            if($filter['columnName'] == 'CAUSE_NAME') {
                $conditions[$key]['columnName'] = 'CAUSE';
            } else if($filter['columnName'] == 'CALL_WITHOUT_LABELS_CHECK') {
                if($filter['value'] == '%Y%') {
                    $specialFilters .= " AND NOT EXISTS(SELECT CLR.CALL FROM CP_CALL_LABEL_RELATION CLR WHERE CLR.CALL = C.NRCALL AND CLR.LABEL IN (SELECT NRCALLLABEL FROM CP_CALL_LABEL WHERE CDOPERINCLUSAO = :USER_ID) OR CLR.LABEL IS NULL)";
                }
                unset($conditions[$key]);
            }  else if($filter['columnName'] == 'CALL_LABELS') {
                $conditions[$key]["columnName"] = "CLR.LABEL";
                $specialFilters .= " AND EXISTS(SELECT CLR.CALL FROM CP_CALL_LABEL_RELATION CLR WHERE CLR.CALL = C.NRCALL AND CLR.LABEL IN(".implode(',',$filter['value'])."))";
                unset($conditions[$key]);
            } else if($filter['columnName'] == 'NRCALL'){
                $conditions[$key]['columnName'] = 'ID';
            } else if($filter['columnName'] == 'STATUS') {
                $situation = $this->situationFilter($conditions[$key]['value']);
                unset($conditions[$key]);
            }
        }

        $conditionsQuery = Utilities::createQueryFiltered($conditions);
        return array($conditionsQuery, $situation['CONTATO'], $specialFilters);
    }

    private function buildWhereFilterToProductCalls($conditions)
    {
        $specialFilters = '';
        $callType = '';
        $situation = array(
            'ISSUE' =>'',
            'CONTATO' =>'',
            'SOLISERV' =>'',
        );
        foreach($conditions as $key => $filter) {
            if($filter['columnName'] == 'PRODUCT_TYPE') {
                $specialFilters .= " AND EXISTS(SELECT * FROM SOLISERV SOLIC WHERE SOLIC.CDPRODGRUP = ";
                $specialFilters .= "'". $filter['value'] . "' AND SOLIC.NRSOLISERV = C.CALL_RELATIONSHIP)";
                unset($conditions[$key]);
            } else if($filter['columnName'] == 'CALL_WITHOUT_LABELS_CHECK') {
                if($filter['value'] == '%Y%') {
                    $userId = Utilities::getEnvironment()->getCurrentUserId();
                    $specialFilters .= " AND NOT EXISTS(SELECT CLR.CALL FROM CP_CALL_LABEL_RELATION CLR WHERE CLR.CALL = C.NRCALL AND CLR.LABEL IN (SELECT NRCALLLABEL FROM CP_CALL_LABEL WHERE CDOPERINCLUSAO = '$userId') OR CLR.LABEL IS NULL)";
                }
                unset($conditions[$key]);
            } else if($filter['columnName'] == 'CALL_LABELS') {
                $conditions[$key]["columnName"] = "CLR.LABEL";
                $specialFilters .= "AND EXISTS(SELECT CLR.CALL FROM CP_CALL_LABEL_RELATION CLR WHERE CLR.CALL = C.NRCALL AND CLR.LABEL IN(".implode(',',$filter['value'])."))";
                unset($conditions[$key]);
            } else if($filter['columnName'] == 'TYPE') {
                $callType = $conditions[$key]['value'];
                unset($conditions[$key]);
            } else if($filter['columnName'] == 'STATUS') {
                $situation = $this->situationFilter($conditions[$key]['value']);
                unset($conditions[$key]);
            } else if($filter['columnName'] == 'NRCALL'){
                $conditions[$key]['columnName'] = 'ID';
            } else if($filter['columnName'] == 'CLASSIFICATION_NAME'){
                $conditions[$key]['columnName'] = 'CLASSIFICATION';
            }
        }
        $conditionsQuery = DevOpsUtilities::createQueryFiltered($conditions);

        return array($callType, $conditionsQuery, $situation, $specialFilters);
    }

    private function findMyProductCalls($conditions, $orderBy = array(), $page = 1, $pageSize = 999999)
    {
        $finalPos   = $pageSize*$page;
        $startPos   = $finalPos-($pageSize-1);
        $cdCliente  = $this->environment->getCurrentCdCliente();
        $userId     = $this->environment->getCurrentUserId();
        $cpCallRepository = InstanceProvider::getRepository()->getCpCall();
        list($callType, $conditionsQuery, $situation, $specialFilters) = $this->buildWhereFilterToProductCalls($conditions);
        $issueTypeProductCalls = array();
        $richClientTypeProductCalls = array();
        if($orderBy){
            $this->orderReverse = current($orderBy) == 'ASC';
            $orderByName = str_replace('_AT','_ORDER',key($orderBy));
            $this->orderBy = $orderByName;
        }
        $situationFilters = $this->getAllSituationDividedByType();
        if($callType == \Service\Call::TYPE_ISSUE) {
            $issueTypeProductCalls = $cpCallRepository->findMyIssueTypeProductCalls($cdCliente, $conditionsQuery, $specialFilters,
                $situation['ISSUE'],  $userId, $situationFilters, $startPos, $finalPos);
        } else if ($callType == \Service\Call::TYPE_RICH_CLIENT) {
            $richClientTypeProductCalls = $cpCallRepository->findMyRichClientTypeProductCalls($cdCliente,
                $conditionsQuery, $specialFilters, $situation['SOLISERV'],$userId, $situationFilters, $startPos, $finalPos);
        } else {
            $issueTypeProductCalls = $cpCallRepository->findMyIssueTypeProductCalls($cdCliente, $conditionsQuery, $specialFilters,
                $situation['ISSUE'], $userId, $situationFilters, $startPos, $finalPos);

            $richClientTypeProductCalls = $cpCallRepository->findMyRichClientTypeProductCalls($cdCliente,
                $conditionsQuery, $specialFilters, $situation['SOLISERV'],$userId, $situationFilters, $startPos, $finalPos);
        }
        $productCalls = array_merge($issueTypeProductCalls, $richClientTypeProductCalls);
        usort($productCalls, function ($product1, $product2) {
            if($this->orderReverse){
                return $product1[$this->orderBy] > $product2[$this->orderBy];
            }else{
                return $product1[$this->orderBy] < $product2[$this->orderBy];
            }

        });

        return $productCalls;
    }

    public function closeSupportCall($callRelationship, $comment)
    {
        $connection = $this->entityManager->getConnection();
        try {
            $connection->beginTransaction();
            $description = "Chamado encerrado pelo cliente";
            $createdBy = $this->environment->getCurrentUserId();
            /** @var \Teknisa\Model\Crm\Entities\Contato $contato */
            $contato = \Util\Instance\Repository::getContato()->findOneBy(array('nrseqcontato' => $callRelationship));
            $idSituacaoOdd = $contato->getNrseqsituacao();
            $idSituacaoNew = self::CRM_CLOSE;
            $situacoes = \Util\Instance\Repository::getCpCallComment()->getCrmSituacao($idSituacaoOdd,$idSituacaoNew);
            $nameSituacaoNew = null;
            $nameSituacaoOdd = null;
            foreach ($situacoes as $situacao) {
                if($situacao['ID'] == $idSituacaoNew){
                    $nameSituacaoNew = $situacao['NAME'];
                }
                if($situacao['ID'] == $idSituacaoOdd){
                    $nameSituacaoOdd = $situacao['NAME'];
                }
            }

            $serviceCallComment = \Util\Instance\Service::getCallComment();
            $serviceCallComment->saveSupportCallComment($description, $createdBy, $callRelationship);
            $LINE_BREAK = Utilities::LINE_BREAK;
            $description = "Mudança de Status do Ticket: $callRelationship $LINE_BREAK<b>De:</b> $nameSituacaoOdd $LINE_BREAK<b>Para:</b> $nameSituacaoNew";
            $serviceCallComment->saveSupportCallComment($description, 1, $callRelationship,'S','E','R');
            $reference = $contato->getDsrefecont();
            $cdproduct = $contato->getCdproduto();
            $cdopercont = $contato->getCdopercont();
            $classification = $contato->getCdmoticont();
            $descriptionTicket = $contato->getDsaberturacaso();
            /** @var Operador $reporter */
            $reporter =  InstanceProvider::getRepository()->getOperador()
                ->findOneBy(array('nrparcnegocio' => $contato->getNrparcinterlocutor()));
            $this->saveTicketCall($reference, $cdproduct, $descriptionTicket, $callRelationship, $reporter,$classification,
                $idSituacaoNew,$cdopercont);
            $this->entityManager->flush();
            $connection->commit();

        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    public function approvePreBudget($callRelationship, $comment)
    {
        $connection = $this->entityManager->getConnection();
        try {
            $connection->beginTransaction();
            $description = $comment;
            $createdBy = $this->environment->getCurrentUserId();
            /** @var \Teknisa\Model\Crm\Entities\Contato $contato */
            $contato = \Util\Instance\Repository::getContato()->findOneBy(array('nrseqcontato' => $callRelationship));
            $idSituacaoOdd = $contato->getNrseqsituacao();
            $idSituacaoNew = self::CRM_CLOSE;
            $situacoes = \Util\Instance\Repository::getCpCallComment()->getCrmSituacao($idSituacaoOdd,$idSituacaoNew);
            $nameSituacaoNew = null;
            $nameSituacaoOdd = null;
            foreach ($situacoes as $situacao) {
                if($situacao['ID'] == $idSituacaoNew){
                    $nameSituacaoNew = $situacao['NAME'];
                }
                if($situacao['ID'] == $idSituacaoOdd){
                    $nameSituacaoOdd = $situacao['NAME'];
                }
            }

            $serviceCallComment = \Util\Instance\Service::getCallComment();
            $serviceCallComment->saveSupportCallComment($description, $createdBy, $callRelationship);
            $LINE_BREAK = Utilities::LINE_BREAK;
            $description = "Mudança de Status do Ticket: $callRelationship $LINE_BREAK<b>De:</b> $nameSituacaoOdd $LINE_BREAK<b>Para:</b> $nameSituacaoNew";
            $serviceCallComment->saveSupportCallComment($description, 1, $callRelationship,'S','E','R');
            $reference = $contato->getDsrefecont();
            $cdproduct = $contato->getCdproduto();
            $cdopercont = $contato->getCdopercont();
            $classification = $contato->getCdmoticont();
            $descriptionTicket = $contato->getDsaberturacaso();
            /** @var Operador $reporter */
            $reporter =  InstanceProvider::getRepository()->getOperador()
                ->findOneBy(array('nrparcnegocio' => $contato->getNrparcinterlocutor()));
            $productCall = $this->saveTicketCall($reference, $cdproduct, $descriptionTicket, $callRelationship, $reporter,$classification,
                $idSituacaoNew,$cdopercont);
            $this->generationCallDevelopmentByTicket($productCall);


            $this->entityManager->flush();
            $connection->commit();

        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    public function generationCallDevelopmentByTicket( Entities\CpCall $productCall){
        $type = $productCall->getType();
        $callRelationship = $productCall->getCallRelationship();
        $productCall->setTicketRelationship($productCall->getCallRelationship());
        $productRelationship = InstanceProvider::getRepository()->getProduct()->getProductCallDevelopment($productCall->getProduct());
        if($productRelationship && $type == self::TYPE_TICKET) {
            $contatoRepository     = InstanceProvider::getRepository()->getContato();
            /** @var Contato $ticket */
            $ticket = $contatoRepository->findOneBy(array('nrseqcontato'=>$callRelationship));
            /** @var Entities\Moticont $moticont */
            $moticont = InstanceProvider::getRepository()->getMoticont()
                ->findOneBy(array('cdmoticont'=>$ticket->getCdmoticont()));
            $reference = $ticket->getDsrefecont();
            $description = $ticket->getDsaberturacaso();
            /** @var Operador $reporter */
            $reporter = InstanceProvider::getRepository()->getOperador()->findOneBy(array('nrparcnegocio'=>$ticket->getNrparcinterlocutor()));
            $reportedBy = $reporter->getCdoperador();
            $cdCliente = $reporter->getCdcliente();
            if ($productRelationship['PROJECT_ID']) {
                $productCall->setType(self::TYPE_ISSUE);
                $product = $productRelationship['PROJECT_ID'];
                $classification = $moticont->getClassificationId();

                $board = $productRelationship['PROJECT_ID'];

                $callRelationship = $this->createIssueInPM($reference, $product, $classification, $board,
                    $description,$reportedBy,null,$cdCliente);
                $productCall->setCallRelationship($callRelationship);
                $this->entityManager->persist($productCall);
                $this->entityManager->flush();

            } else if ($productRelationship['CDPRODCOMP'] && $productRelationship['CDPRODGRUP']) {

                $product = $productRelationship['CDPRODCOMP'];
                $productType = $productRelationship['CDPRODGRUP'];
                $productVersion = "";
                $classification = $moticont->getCdassunto();
                $productRelease = "";
                $productPatch = "";
                $callRelationship = null;
                $reference = substr($reference, 0, 60);
                $useLicense = "";

                $callRelationship = $this->saveSolicitacao($reference, $product, $productType, $productVersion,
                    $useLicense, $classification, $description, $reportedBy, $productRelease, $productPatch,
                    $cdCliente, null);

                $productCall->setCallRelationship($callRelationship);
                $productCall->setType(self::TYPE_RICH_CLIENT);
                $this->entityManager->persist($productCall);
                $this->entityManager->flush();
            } else {

            }
        }

    }

    public function declinePreBudget($callRelationship, $comment)
    {
        $connection = $this->entityManager->getConnection();
        try {
            $connection->beginTransaction();
            $description = $comment;
            $createdBy = $this->environment->getCurrentUserId();
            /** @var \Teknisa\Model\Crm\Entities\Contato $contato */
            $contato = \Util\Instance\Repository::getContato()->findOneBy(array('nrseqcontato' => $callRelationship));
            $idSituacaoOdd = $contato->getNrseqsituacao();
            $idSituacaoNew = self::CRM_INIC;
            $situacoes = \Util\Instance\Repository::getCpCallComment()->getCrmSituacao($idSituacaoOdd,$idSituacaoNew);
            $nameSituacaoNew = null;
            $nameSituacaoOdd = null;
            foreach ($situacoes as $situacao) {
                if($situacao['ID'] == $idSituacaoNew){
                    $nameSituacaoNew = $situacao['NAME'];
                }
                if($situacao['ID'] == $idSituacaoOdd){
                    $nameSituacaoOdd = $situacao['NAME'];
                }
            }

            $serviceCallComment = \Util\Instance\Service::getCallComment();
            $serviceCallComment->saveSupportCallComment($description, $createdBy, $callRelationship);
            $LINE_BREAK = Utilities::LINE_BREAK;
            $description = "Mudança de Status do Ticket: $callRelationship $LINE_BREAK<b>De:</b> $nameSituacaoOdd $LINE_BREAK<b>Para:</b> $nameSituacaoNew";
            $serviceCallComment->saveSupportCallComment($description, 1, $callRelationship,'S','E','R');
            $reference = $contato->getDsrefecont();
            $cdproduct = $contato->getCdproduto();
            $cdopercont = $contato->getCdopercont();
            $classification = $contato->getCdmoticont();
            $descriptionTicket = $contato->getDsaberturacaso();
            /** @var Operador $reporter */
            $reporter =  InstanceProvider::getRepository()->getOperador()
                ->findOneBy(array('nrparcnegocio' => $contato->getNrparcinterlocutor()));
            $this->saveTicketCall($reference, $cdproduct, $descriptionTicket, $callRelationship,$reporter,$classification,
                $idSituacaoNew,$cdopercont,true);
            $this->entityManager->flush();
            $connection->commit();

        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    /**
     * @param \Teknisa\Model\Teknisa\Entities\Soliserv  $soliserv
     * @return null |\Teknisa\Model\Teknisa\Entities\Licencauso
     */
    private function getLicencausoBySoliserv(\Teknisa\Model\Teknisa\Entities\Soliserv $soliserv = null)
    {
        $licencauso = null;
        if ($soliserv->getCdsetor() != null) {
            $id = array(
                "cdcliente" => $soliserv->getCdcliente(),
                "nrlicencauso" => $soliserv->getNrlicencauso(),
                "nrorg" => $soliserv->getNrorg(),
            );
            $licencauso = InstanceProvider::getRepository()->getLicencauso()->findOneBy($id);
        }
        return $licencauso;
    }

    private function findProductCall($callRelationship, $callType)
    {
        $cpCallRepository   = InstanceProvider::getRepository()->getCpCall();
        $soliservRepository = InstanceProvider::getRepository()->getSoliserv();
        $userId             = $this->environment->getCurrentUserId();
        $productCall        = array();

        $situationFilters = $this->getAllSituationDividedByType();
        /** @var Entities\CpCall $call */
        $call = $cpCallRepository->findOneBy(array('callRelationship' => $callRelationship));
        if(!empty($call)){
            if($call->getType() == \Service\Call::TYPE_ISSUE) {
                $productCall = $this->findCallTypeIssue($call->getNrcall());
                $productCall['TYPE_LABEL'] = $this->getCallTypeDescription($call->getType());
                if(empty($productCall['CALL_LABELS'])){
                    $productCall['CALL_LABELS'] = array();
                } else {
                    $productCall['CALL_LABELS'] = explode(',', $productCall['CALL_LABELS']);
                }

            } else if ($call->getType() == \Service\Call::TYPE_RICH_CLIENT) {
                $productCall = $this->findCallTypeRichClient($call->getNrcall(), $userId);
                $productCall['DESCRIPTION'] = Utilities::rtf2textPlain($productCall['DESCRIPTION'], true);
                $productCall['TYPE_LABEL'] = $this->getCallTypeDescription($call->getType());
                if(empty($productCall['CALL_LABELS'])){
                    $productCall['CALL_LABELS'] = array();
                } else {
                    $productCall['CALL_LABELS'] = explode(',', $productCall['CALL_LABELS']);
                }

                $soliServ = $soliservRepository->findOneBy(array('nrsoliserv' => $productCall['CALL_RELATIONSHIP']));

                $useLicense = $this->getLicencausoBySoliserv($soliServ);
                if($useLicense){
                    $productCall['USE_LICENSE'] = $useLicense->getNrlicencauso();
                    $productCall['USE_LICENSE_LABEL'] = $productCall['USE_LICENSE'] . ' - ' . $useLicense->getNmfantluso();
                }else{
                    $productCall['USE_LICENSE'] = '';
                    $productCall['USE_LICENSE_LABEL'] = '';
                }
            }
        } else if ($callType == "I"){
            $productCall = $cpCallRepository->findIssueById($callRelationship, $situationFilters);
            $productCall['CALL_RELATIONSHIP'] = $productCall['ID'];
            $productCall['ID'] = null;
            $productCall['TYPE'] = $callType;
            $productCall['TYPE_LABEL'] = $this->getCallTypeDescription($callType);
            $productCall['CALL_LABELS'] = empty($productCall['CALL_LABELS']) ? array() : explode(',', $productCall['CALL_LABELS']);
        } else if ($callType == "S"){
            $productCall = $cpCallRepository->findSoliservById($callRelationship, $situationFilters, $userId);
            $productCall['DESCRIPTION'] = Utilities::rtf2textPlain($productCall['DESCRIPTION'], true);
            $productCall['CALL_RELATIONSHIP'] = $productCall['ID'];
            $productCall['TYPE'] = $callType;
            $productCall['TYPE_LABEL'] = $this->getCallTypeDescription($callType);
            $productCall['ID'] = null;


            $soliServ = $soliservRepository->findOneBy(array('nrsoliserv' => $productCall['CALL_RELATIONSHIP']));
            $useLicense = $this->getLicencausoBySoliserv($soliServ);

            $productCall['USE_LICENSE'] = $useLicense ? $useLicense->getNrlicencauso() : '';
            $productCall['USE_LICENSE_LABEL'] = $useLicense ? $productCall['USE_LICENSE'] . ' - ' . $useLicense->getNmfantluso() : '';

        }

        if(!empty($productCall)){
            $productCall['MAIN_TYPE_CALL'] = self::MAIN_TYPE_OF_PRODUCT_CALL;
        }

        return $productCall;
    }

    public function findMyMostRecentlyUpdatedCalls($numberOfProductCalls)
    {

        $page = 1;
        $pageSize = $numberOfProductCalls;
        $userId = $this->environment->getCurrentUserId();
        $cdCliente = $this->environment->getCurrentCdCliente();

        $conditions = array(
            [
                'columnName' => 'REPORTED_BY',
                'operator' => 'IN',
                'value' => [$userId]
            ],
            [
                'columnName' => 'STATUS',
                'operator' => 'IN',
                'value' => ['O','P','W']
            ],
            [
                'columnName' => 'CDCLIENTE',
                'operator' => 'IN',
                'value' => [$cdCliente]
            ]
        );

        $orderBy = array("UPDATED_AT" => "DESC");

        return $this->findCalls("P", $conditions, $orderBy, $page, $pageSize);
    }

    public function operadorByResponse($callId, $callType, $cdfastramdes, $nrfastramdes, $callRelationship)
    {
        $cpCallRepository   = InstanceProvider::getRepository()->getCpCall();
        $soliservRepository = InstanceProvider::getRepository()->getSoliserv();

        if($callId){
            /** @var Entities\CpCall $call */
            $call = $cpCallRepository->find($callId);
            $callRelationship = $call->getCallRelationship();
        }
        if($callType == self::TYPE_ISSUE){
            $return = $cpCallRepository->operadorByResponseIssue($callRelationship);
        }else{
            $return = [];
            if($cdfastramdes && $nrfastramdes){
                /** @var Entities\Soliserv $soliserv */
                $soliserv = $soliservRepository->findOneBy(array( 'nrsoliserv' => $callRelationship));
                list($iddepacaoope,$cdgroperresp) = $cpCallRepository->operadorGrupoByResponse($soliserv->getCdfasetram(),
                    $cdfastramdes, $nrfastramdes);
                $cdCliente = $this->environment->getCurrentCdCliente();
                $return = $cpCallRepository->operadorByResponse($iddepacaoope,$cdgroperresp,$cdCliente);
            }
        }
        return $return;

    }

    public function findProductCallClassificationFromRichClient($params)
    {
        $filterString = DevOpsUtilities::createQueryFiltered($params);
        return InstanceProvider::getRepository()->getAssunto()->findProductCallClassificationFromRichClient($filterString);
    }

    public function sendCallGep($nrcall, $cdfastramdes, $cdoperador, $description, $nrfastramdes,
                                $txacaofatram, $callType = null, $callRelationship = null, $documents = array(), $documentFileServer = array())
    {
        $cpCallRepository   = InstanceProvider::getRepository()->getCpCall();
        $soliservRepository = InstanceProvider::getRepository()->getSoliserv();
        $connection = $this->entityManager->getConnection();
        try {
            $connection->beginTransaction();
            /** @var \Teknisa\Model\Teknisa\Entities\CpCall $productCall */
            $productCall = $cpCallRepository->findOneBy(array('callRelationship'=>$callRelationship));
            if($productCall){
                $nrcall = $productCall->getNrcall();
            } else {
                if($callType == Call::TYPE_ISSUE){
                    $issue = $cpCallRepository->findIssueById($callRelationship);
                    $productCall = $this->saveNewIssueTypeProductCall($callType, $issue['REFERENCE'], $issue['PRODUCT'],
                    $issue['CLASSIFICATION'], $issue['BOARD'], $issue['DESCRIPTION'], null, array(), $callRelationship);
                } else if ($callType == Call::TYPE_RICH_CLIENT){
                    $userId           = $this->environment->getCurrentUserId();
                    $situationFilters = InstanceProvider::getService()->getCall()->getAllSituationDividedByType();
                    $soliserv         = $cpCallRepository->findSoliservById($callRelationship, $situationFilters, $userId);
                    $descriptionInTextPlain = Utilities::rtf2textPlain($soliserv['DESCRIPTION']);
                    $productCall = $this->saveNewRichClientTypeProductCall($callType, $soliserv['REFERENCE'],
                                    $soliserv['PRODUCT'], $soliserv['PRODUCT_TYPE'], $soliserv['PRODUCT_VERSION'],
                                    $soliserv['USE_LICENSE'], $soliserv['CLASSIFICATION'], $descriptionInTextPlain, $soliserv['REPORTER_BY'],
                                    array(), $soliserv['PRODUCT_RELEASE'], $soliserv['PRODUCT_PATCH'], $callRelationship);
                }
                $nrcall = $productCall['ID'];
            }


            /** @var \Teknisa\Model\Teknisa\Entities\CpCall $productCall */
            $productCall = $cpCallRepository->find($nrcall);
            if ($productCall->getType() == \Service\Call::TYPE_RICH_CLIENT) {
                $paramGeral         = $cpCallRepository->getallParamGeral($productCall->getNrorg());
                $cdCliente          = $this->environment->getCurrentCdCliente();
                $cdOperadorCurrent  = $this->environment->getCurrentUserId();
                $nrsoliserv         = $productCall->getCallRelationship();
                $integration        = $cpCallRepository->getIntegrationValidationSoliserv($nrsoliserv);
                /** @var Entities\Soliserv $soliserv */
                $soliserv = $soliservRepository->findOneBy(array('nrsoliserv' => $nrsoliserv));

                $soliserv->setCdfasetram($cdfastramdes);
                if ($soliserv->getCdfasetram() == $paramGeral['CDFATRAMENCE']) {
                    if($soliserv->getCdoperresp() == $cdOperadorCurrent){
                        $soliserv->setIdsolilida('S');
                    }else{
                        $soliserv->setIdsolilida('N');
                    }
                    $soliserv->setIdsolience('S');
                    $soliserv->setIdstatussoli('C');
                    $soliserv->setCdoperresp($cdOperadorCurrent);
                    $now = new \DateTime();
                    $soliserv->setDtencesoli($now);
                }else{
                    if($soliserv->getCdoperresp() == $cdoperador){
                        $soliserv->setIdsolilida('S');
                    }else{
                        $soliserv->setIdsolilida('N');
                    }
                    $soliserv->setCdoperresp($cdoperador);
                }
                $this->entityManager->persist($soliserv);


                $dt             = new \Setforms2\Shared\ORM\DateTime();
                $nrorg          = $productCall->getNrorg();
                $cdfilial       = $paramGeral['CDFILIAL'];
                $cdFaseTram     = $cdfastramdes;
                $cdOperResp     = $soliserv->getCdoperresp();
                $txacaolftram   = $txacaofatram;
                $dsAlFaseTram   = $description;
                $logFaseTram    = InstanceProvider::getService()->getIntranetIntegration()->
                    logFaseTramFactory->createLogFaseTram($nrorg, $cdfilial, $nrsoliserv, $dt,
                        $cdFaseTram, $cdOperadorCurrent, $cdOperResp, $txacaolftram, $dsAlFaseTram);
                $this->entityManager->persist($logFaseTram);
                $dateString = $dt->format('YmdHis');
                InstanceProvider::getService()->getIntranetIntegration()->insertFile($documents, $productCall, $dateString);
                if($nrfastramdes && !empty($integration)){
                    if($nrfastramdes == '01'){
                        SMSoliservIntegration::integrationSMAccept($nrsoliserv,$productCall->getNrorg());
                        $budgetRepository = InstanceProvider::getRepository()->getBudget();
                        $nrseqcontrato = $budgetRepository->findNrseqcontratoWithSoliservId($nrsoliserv)[0]["NRSEQCONTRATO"];
                        $connection = $this->entityManager->getConnection();
                        $connection->beginTransaction();
                        foreach($documentFileServer as $doc){
                            $newCodeService = InstanceProvider::getService()->getNewCode();
                            $fctAnexocontratoFactory      = InstanceProvider::getFactory()->getFctAnexoContrato();
                            $dt       = new \DateTime();
                            $operador = $this->environment->getCurrentUser();
                            $nrorg      = $operador->getNrorg();
                            $operador = $operador->getCdoperador();
                            $nranexocontrato = $newCodeService->getCodeByTableName('FCT_ANEXOCONTRATO', $nrorg, 1, 10, true );
                            try {
                                
                                $anexoEntity = $fctAnexocontratoFactory->linkDocumentsToContract($nranexocontrato, $nrseqcontrato, $doc["name"], $doc["path"], $doc["type"], $doc["size"], $nrorg, $operador, $dt);
                                $this->entityManager->persist($anexoEntity);
                           
                                $this->entityManager->flush();
                                
                                } catch (\Exception $e) {
                                $connection->rollBack();
                                throw $e;
                            }
                        }
                        $connection->commit();
                    }else{
                        SMSoliservIntegration::integrationSMCancel($nrsoliserv,$productCall->getNrorg());
                    }
                }

                $this->entityManager->flush();

                $connection->commit();
            }
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    public function getQuestionForSendCall($callId, $lang, $callType, $callRelationship = null)
    {
        $cpCallRepository   = InstanceProvider::getRepository()->getCpCall();
        $soliservRepository = InstanceProvider::getRepository()->getSoliserv();
        $TaskRepository = InstanceProvider::getRepository()->getTask();
        if($callId){
            /** @var Entities\CpCall $call */
            $call = $cpCallRepository->find($callId);
            $callRelationship = $call->getCallRelationship();
        }
        $nrorg = Utilities::getEnvironment()->getCurrentOrganizationId();
        $cdfasetram = '';
        $unique = false;
        if($callType == self::TYPE_ISSUE){
            /** @var Task $issue */
            $issue = $TaskRepository->findOneBy(array( 'id' => $callRelationship));
            $controlStatus = $issue->getTaskStatus()->getControl();
            $controlType = $issue->getTaskStatus()->getTaskType()->getControl();
            if($controlStatus == TaskStatus::TASK_STATUS_CONTROL_CLIENT_ACCEPT){
                $cdfasetram = self::STAGE_ESPEC;
            } else if($controlType == TaskType::CONTROL_HOMOLOGATED){
                $cdfasetram = self::STAGE_RETUN;
            } else {
                $cdfasetram = self::STAGE_IMFOR;
                $unique = true;
            }
        }else{
            /** @var Entities\Soliserv $soliserv */
            $soliserv = $soliservRepository->findOneBy(array( 'nrsoliserv' => $callRelationship));
            $cdfasetram = $soliserv->getCdfasetram();
        }
        $return = $cpCallRepository->getInfoForSendCall($cdfasetram, Utilities::langForIdidioma($lang),$nrorg);
        if($unique){
            $return = array($return[0]);
            $return[0]['IDOBSOBRTRAM'] = 'Y';
        }
        return $return;
    }

    public function getInfoForSendCall($callId, $lang)
    {
        $cpCallRepository   = InstanceProvider::getRepository()->getCpCall();
        $soliservRepository = InstanceProvider::getRepository()->getSoliserv();
        /** @var Entities\CpCall $call */
        $call = $cpCallRepository->find($callId);
        /** @var Entities\Soliserv $soliserv */
        $soliserv = $soliservRepository->findOneBy(array( 'nrsoliserv' => $call->getCallRelationship()));
        $nrorg = Utilities::getEnvironment()->getCurrentOrganizationId();
        return $cpCallRepository->getInfoForSendCall($soliserv->getCdfasetram(),Utilities::langForIdidioma($lang),$nrorg);
    }

    public function findProductCallProductsFromTicket($params)
    {
        $cpCallRepository   = InstanceProvider::getRepository()->getCpCall();
        $filterString = DevOpsUtilities::createQueryFiltered($params);
        return $cpCallRepository->findProductCallProductsFromTicket($filterString);
    }

    public function findProductCallProductsFromRichClient($productType, $params)
    {
        $filterString = DevOpsUtilities::createQueryFiltered($params);
        return $this->getSistema($productType, '', '', '', $filterString);
    }

    private function getSistema($P_CDGRUPSIST = '', $P_CDSISTEMA = '', $P_NMSISTEMA = '', $P_TIPO = '', $filterString = '')
    {

        $adqSubProd = "
            SELECT
               DISTINCT(PR.CDARVPROD), PR.CDPRODUTO AS ID, PR.NMPRODUTO AS NAME
            FROM
               PARFILIALGEP PG, PRSUBGRPROD NV, PRODUTO PR,
               (SELECT CDARVPROD, CDPRODUTO FROM PRODUTO WHERE CDPRODUTO = NVL(:P_CDPRODGRUP,CDPRODUTO)) GRUP
            WHERE
                (PG.CDFILIAL = '9999')
                AND (NV.CDFILISUBGR = '9999' OR NV.CDFILISUBGR = 'P')
                AND (PG.CDGRUPPROD = NV.CDGRUPPROD)
                AND (NV.CDPRODUTO = PR.CDPRODUTO)
                AND (PR.CDARVPROD LIKE GRUP.CDARVPROD||'%')
                AND (PR.CDARVPROD = NVL(:P_CDARVPROD,PR.CDARVPROD))
                AND (INSTR (:P_CDPRODUTO, GRUP.CDPRODUTO) > 0)
        ";

        $adqSubProdParams = array(
            'P_CDPRODGRUP' => '',
            'P_CDARVPROD' => '',
            'P_CDPRODUTO' => ''
        );

        $adqProdGrupInt = "
            SELECT
               DISTINCT(PR.CDARVPROD), PR.CDPRODUTO AS ID, PR.NMPRODUTO AS NAME, PR.DSDIRBASEVER
            FROM
               PRODUTO PR,  NVSUBGRPROD NS, PARFILIALGEP PG
            WHERE
               (NS.CDGRUPPROD  = PG.CDGRUPPROD ) AND
               (NS.CDPRODUTO   = PR.CDPRODUTO  ) AND
               (PG.CDFILIAL    =    :P_CDFILIAL) AND
               (PR.CDARVPROD   = NVL(:P_CDARVPROD, PR.CDARVPROD)) AND
               (NS.CDFILISUBGR IN ('P', :P_CDFILIAL))
        ";

        $adqProdGrupIntParams = array(
            'P_CDFILIAL' => '',
            'P_CDARVPROD' => '',
        );

        $adqProdGrup = "
            SELECT CDPRODUTO
              FROM PRODUTO
             WHERE (SUBSTR(CDARVPROD,1,1) = 'P')
               AND (CDNVPRODUTO = :P_NVPRODUTO)
               AND (CDPRODUTO <> :P_CDPRODFIL)
        ";

        $adqProdGrupParams = array(
            'P_NVPRODUTO' => '',
            'P_CDPRODFIL' => ''
        );

        $stProdGrup = '';
        $cdProdGrup = '';
        try {
            if($P_TIPO == '') {
                if(!empty($P_CDSISTEMA) && !empty($P_NMSISTEMA)){
                    $adqSubProd = "SELECT * FROM ( " . $adqSubProd;
                    $adqSubProd .= " AND UPPER(PR." . $P_CDSISTEMA .") LIKE UPPER('" . $P_NMSISTEMA . '%' ."') ) WHERE 1 = 1 ". $filterString . "ORDER BY " . $P_CDSISTEMA;
                } else {
                    $adqSubProd = "SELECT * FROM ( " . $adqSubProd;
                    $adqSubProd .= ")\n WHERE 1 = 1" . $filterString;
                    $adqSubProd .= " ORDER BY CDARVPROD";
                }

                $cdGrupSist  = strtoupper($P_CDGRUPSIST);
                if($cdGrupSist != 'DOS' && $cdGrupSist != 'WEB' && $cdGrupSist != 'T') {
                    $stProdGrup = '_' . $P_CDGRUPSIST . '_';
                    $cdProdGrup = $P_CDGRUPSIST;
                } else {
                    $cdGrupSist  = strtoupper($P_CDGRUPSIST);

                    if ($cdGrupSist == 'DOS' ){
                        $stProdGrup = '_0000000102_';
                        $cdProdGrup = '0000000102';
                    }

                    if ($cdGrupSist == 'WEB' ){
                        $stProdGrup = '_0000000627_';
                        $cdProdGrup = '0000000627';
                    }

                    if ($cdGrupSist == 'T' ){
                        $cdProdGrup = '';
                        $stProdGrup = '';


                        /* adqProdGrup.ParamByName('P_NVPRODUTO').AsString := IntToStr(TTNiveis(ptProduto) - 1); */
                        $adqProdGrup['P_NVPRODUTO'] = '';
                        $adqProdGrup['P_CDPRODFIL'] = ' ';
                        $adqProdGrupResult = $this->entityManager->getConnection()->fetchAll($adqProdGrup, $adqProdGrupParams);

                        foreach($adqProdGrupResult as $adqProdGrupRow) {
                            $stProdGrup = $stProdGrup . '_' . $adqProdGrupRow['CDPRODUTO'];
                        }
                        $stProdGrup = $stProdGrup . '_';
                    }
                }

                $adqSubProdParams['P_CDPRODUTO'] = $stProdGrup;
                $adqSubProdParams['P_CDPRODGRUP'] = $cdProdGrup;
                $adqSubProdParams['P_CDARVPROD'] = '';

                $adqSubProdResults = $this->entityManager->getConnection()->fetchAll($adqSubProd, $adqSubProdParams);
                return $adqSubProdResults;
            } else {
                if(!empty($P_CDSISTEMA) && !empty($P_NMSISTEMA)){
                    $adqProdGrupInt = "SELECT * FROM ( " .  $adqProdGrupInt;
                    $adqProdGrupInt .= " AND UPPER(PR." . $P_CDSISTEMA . ") LIKE UPPER('" . $P_NMSISTEMA . '%' ."') ) WHERE 1 = 1" . $filterString . "ORDER BY " . $P_CDSISTEMA;
                } else {
                    $adqProdGrupInt = "SELECT * FROM ( " .  $adqProdGrupInt;
                    $adqProdGrupInt .= ")\n WHERE 1 = 1" . $filterString;
                    $adqProdGrupInt .= " ORDER BY PR.CDARVPROD";
                }

                $adqProdGrupInt['P_CDARVPROD'] = '';
                $adqProdGrupInt['P_CDFILIAL'] = '9999';

                $adqProdGrupIntResults = $this->entityManager->getConnection()->fetchAll($adqProdGrupInt, $adqProdGrupIntParams);
                return $adqProdGrupIntResults;
            }
        } catch(\Exception $e) {
            return array();
        }
    }

    public function findCallTypeCaso($id,$situationFilters)
    {
        $cpCallRepository = InstanceProvider::getRepository()->getCpCall();
        return $cpCallRepository->findCallTypeCaso($id,$situationFilters);
    }

    private function findSupportCall($callRelationship = null)
    {

        $situationFilters = $this->getAllSituationDividedByType();
        $productCall = $this->findCallTypeCaso($callRelationship,$situationFilters);
        $productCall['TYPE_LABEL'] = $this->getCallTypeDescription($productCall['TYPE']);
        if(empty($productCall['CALL_LABELS'])){
            $productCall['CALL_LABELS'] = array();
        } else {
            $productCall['CALL_LABELS'] = explode(',', $productCall['CALL_LABELS']);
        }
        return $productCall;
    }

    public function getCall( $callRelationship, $callType)
    {
        $call = array();
        if($callType == self::TYPE_RICH_CLIENT || $callType == self::TYPE_ISSUE){
            $call = $this->findProductCall($callRelationship, $callType);
        }else if($callType == self::TYPE_TICKET){
            $call = $this->findSupportCall($callRelationship);
        }

        return $call;
    }

    public function closeProductCall($nrcall,$callRelationship,$reference,$product,$classification,$board,
                                     $description, $comment, $documents)
    {
        $cpCallRepository = InstanceProvider::getRepository()->getCpCall();
        $taskRepository = InstanceProvider::getRepository()->getTask();
        /** @var \Teknisa\Model\Teknisa\Entities\CpCall $productCall */
        if(!$nrcall && $callRelationship){
            $productCallId = InstanceProvider::getService()->getCall()->saveNewIssueTypeProductCall(\Service\Call::TYPE_ISSUE, $reference, $product,
                $classification, $board, $description, null, array(), $callRelationship);
            $productCall = $cpCallRepository->findOneBy(array('nrcall' => $productCallId['ID']));
        }else{
            $productCall = $cpCallRepository->findOneBy(array('nrcall' => $nrcall));
        }
        if($productCall->getType() == \Service\Call::TYPE_ISSUE){
            InstanceProvider::getService()->getAttachment()->uploadCallAttachments($nrcall, $comment, $documents,
                $this->environment->getCurrentUser()->getNmoperador(), $callRelationship, null, 'I');
            $closeIssueTaskStatus = Utilities::getCloseIssueTaskStatus();
            /** @var \Zeedhi\DevOps\Model\Projectmanagement\Entities\Task $issue */
            $issue = $taskRepository->find($productCall->getCallRelationship());
            $issue->setTaskStatus($closeIssueTaskStatus);
            $this->entityManager->persist($issue);
            $this->entityManager->flush();
        }
    }

    public function reopenIssue($nrcall,$comment,$callRelationship,$reference,$product,$classification,$board,
                                $description,$documents)
    {
        $cpCallRepository = InstanceProvider::getRepository()->getCpCall();
        /** @var \Teknisa\Model\Teknisa\Entities\CpCall $productCall */
        if(!$nrcall && $callRelationship){
            $productCallId = InstanceProvider::getService()->getCall()->saveNewIssueTypeProductCall(\Service\Call::TYPE_ISSUE, $reference, $product,
                $classification, $board, $description, null, array(), $callRelationship);
            $productCall = $cpCallRepository->findOneBy(array('nrcall' => $productCallId['ID']));
        }else{
            $productCall = $cpCallRepository->findOneBy(array('nrcall' => $nrcall));
        }
        if($productCall->getType() == \Service\Call::TYPE_ISSUE){
            InstanceProvider::getService()->getPmIntegration()->saveReopenIssueByProductCall($productCall->getCallRelationship(),
                $comment, Utilities::getEnvironment()->getCurrentUserId());
            if(!empty($documents)){
                InstanceProvider::getService()->getAttachment()->uploadCallAttachments($nrcall, $comment, $documents,
                    $this->environment->getCurrentUser()->getNmoperador(), $callRelationship, null, 'I', false);
            }
        }
    }

    public function refuseIssue($nrcall,$comment,$callRelationship,$reference,$product,$classification,$board,
                                $description,$documents)
    {
        $cpCallRepository = InstanceProvider::getRepository()->getCpCall();
        /** @var \Teknisa\Model\Teknisa\Entities\CpCall $productCall */
        if(!$nrcall && $callRelationship){
            $productCallId = InstanceProvider::getService()->getCall()->saveNewIssueTypeProductCall(\Service\Call::TYPE_ISSUE, $reference, $product,
                $classification, $board, $description, null, array(), $callRelationship);
            $productCall = $cpCallRepository->findOneBy(array('nrcall' => $productCallId['ID']));
        }else{
            $productCall = $cpCallRepository->findOneBy(array('nrcall' => $nrcall));
        }
        if($productCall->getType() == \Service\Call::TYPE_ISSUE){
            InstanceProvider::getService()->getPmIntegration()->saveRefuseIssueByProductCall($productCall->getCallRelationship(),
                $comment, Utilities::getEnvironment()->getCurrentUserId());
            if(!empty($documents)){
                InstanceProvider::getService()->getAttachment()->uploadCallAttachments($nrcall, $comment, $documents,
                    $this->environment->getCurrentUser()->getNmoperador(), $callRelationship, null, 'I', false);
            }
        }
    }


    public function approveIssue($nrcall,$comment,$callRelationship,$reference,$product,$classification,$board,
                                 $description,$document = null)
    {
        $cpCallRepository = InstanceProvider::getRepository()->getCpCall();
        /** @var \Teknisa\Model\Teknisa\Entities\CpCall $productCall */
        if(!$nrcall && $callRelationship){
            $productCallId = InstanceProvider::getService()->getCall()->saveNewIssueTypeProductCall(\Service\Call::TYPE_ISSUE, $reference, $product,
                $classification, $board, $description, null, array(), $callRelationship);
            $productCall = $cpCallRepository->findOneBy(array('nrcall' => $productCallId['ID']));
        }else{  
            $productCall = $cpCallRepository->findOneBy(array('nrcall' => $nrcall));
        }
        if($productCall->getType() == \Service\Call::TYPE_ISSUE){
            InstanceProvider::getService()->getPmIntegration()->saveApproveIssueByProductCall($productCall->getCallRelationship(),
                $comment, Utilities::getEnvironment()->getCurrentUserId(),$document);
                $nrseqcontrato =  $cpCallRepository->findNrseqcontratoIdWithIssueId($productCall->getCallRelationship())[0]["NRSEQCONTRATO"];
                $connection = $this->entityManager->getConnection();
                $connection->beginTransaction();
                foreach($document as $doc){
                    $newCodeService = InstanceProvider::getService()->getNewCode();
                    $fctAnexocontratoFactory      = InstanceProvider::getFactory()->getFctAnexoContrato();
                    $dt       = new \DateTime();
                    $operador = $this->environment->getCurrentUser();
                    $nrorg      = $operador->getNrorg();
                    $operador = $operador->getCdoperador();
                    $nranexocontrato = $newCodeService->getCodeByTableName('FCT_ANEXOCONTRATO', $nrorg, 1, 10, true );
                    try {
                        
                        $anexoEntity = $fctAnexocontratoFactory->linkDocumentsToContract($nranexocontrato, $nrseqcontrato, $doc["name"], $doc["path"], $doc["type"], $doc["size"], $nrorg, $operador, $dt);
                        $this->entityManager->persist($anexoEntity);
                   
                        $this->entityManager->flush();
                        
                        } catch (\Exception $e) {
                        $connection->rollBack();
                        throw $e;
                    }
                }
                    $connection->commit();
            }
    }

    public function giveUpImplementationIssue($nrcall,$comment,$callRelationship,$reference,$product,$classification,$board,
                                 $description,$document = null)
    {
        $cpCallRepository = InstanceProvider::getRepository()->getCpCall();
        /** @var \Teknisa\Model\Teknisa\Entities\CpCall $productCall */
        if(!$nrcall && $callRelationship){
            $productCallId = InstanceProvider::getService()->getCall()->saveNewIssueTypeProductCall(\Service\Call::TYPE_ISSUE, $reference, $product,
                $classification, $board, $description, null, array(), $callRelationship);
            $productCall = $cpCallRepository->findOneBy(array('nrcall' => $productCallId['ID']));
        }else{
            $productCall = $cpCallRepository->findOneBy(array('nrcall' => $nrcall));
        }
        if($productCall->getType() == \Service\Call::TYPE_ISSUE){
            InstanceProvider::getService()->getPmIntegration()->giveUpImplementationIssue($productCall->getCallRelationship(),
                $comment, Utilities::getEnvironment()->getCurrentUserId(),$document);
        }
    }

    public function findProductCalProductPatch($productType, $version, $release)
    {
        $cpCallRepository = InstanceProvider::getRepository()->getCpCall();
        $productVersion = $cpCallRepository->getPatch($productType, $version, $release);
        return empty($productVersion) ? array(array('ID' => '0', 'LABEL' => '0')) : $productVersion;
    }

    public function findProductCalProductVersions($productType, $product)
    {
        $productVersion = $this->getVersao($productType, $product);
        return empty($productVersion) ? array(array('ID' => '0', 'LABEL' => '0')) : $productVersion;
    }

    public function findProductCallProductReleases($productType, $version)
    {
        $cpCallRepository = InstanceProvider::getRepository()->getCpCall();
        $productVersion = $cpCallRepository->getReleases($productType, $version);
        return empty($productVersion) ? array(array('ID' => '0', 'LABEL' => '0')) : $productVersion;
    }

    private function getVersao($P_CDGRUPSIST, $P_CDSISTEMA, $P_CDVERSGSIS = '', $P_DSVERSGSIS = '')
    {
        $adqPesqVersao = "
            SELECT PR.CDVERPRODUTO AS ID, PR.CDVERPRODUTO AS LABEL
            FROM PRODLIBEVERS PR
            WHERE
               (PR.CDPRODGRUP   = :P_CDPRODGRUP  ) AND
               (PR.CDPRODCOMP   = :P_CDPRODCOMP  ) AND
               (PR.CDVERPRODUTO = NVL(:P_CDVERPRODUTO, PR.CDVERPRODUTO))
        ";

        $adqPesqVersaoParams = array(
            'P_CDPRODGRUP' => '',
            'P_CDPRODCOMP' => '',
            'P_CDVERPRODUTO' => ''
        );

        if(!empty($P_CDVERSGSIS) && !empty($P_DSVERSGSIS)) {
            $adqPesqVersao .= "AND UPPER(". $P_CDVERSGSIS .") LIKE UPPER('" . $P_DSVERSGSIS . "%') ORDER BY " . $P_CDVERSGSIS;
        } else {
            $adqPesqVersao .= 'ORDER BY CDVERPRODUTO';
        }

        $adqPesqVersaoParams['P_CDPRODGRUP'] = $P_CDGRUPSIST;
        $adqPesqVersaoParams['P_CDPRODCOMP'] = $P_CDSISTEMA;
        $adqPesqVersaoParams['P_CDVERPRODUTO'] = '';
        return $this->entityManager->getConnection()->fetchAll($adqPesqVersao, $adqPesqVersaoParams);
    }

    private function validateCommentDescription(Task $issue,$menuVersionId,$menuItemId){
        $validate = false;
        if($menuVersionId || $menuItemId){
            if($issue->getMenuItem() != $menuItemId){
                $validate = true;
            }else if($issue->getProductVersion() && $issue->getProductVersion()->getId() != $menuVersionId ){
                $validate = true;
            }else if(!$issue->getProductVersion() && !$menuVersionId){
                $validate = true;
            }
        }
        return $validate;
    }


    private function updateInvisibleFieldsInCall($id, $invisibleFields){
        $cpCallRepository = InstanceProvider::getRepository()->getCpCall();
        $callFactory      = InstanceProvider::getFactory()->getCall();

        /** @var \Teknisa\Model\Teknisa\Entities\CpCall $productCall */
        $productCall = $cpCallRepository->find($id);

        for($i=1;$i<6;$i++){
            if($invisibleFields["DATE_" . $i]){
                $invisibleFields["DATE_" . $i] = \Util\DateUtil::getDataDeString($invisibleFields["DATE_" . $i],\Util\DateUtil::FORMATO_BRASILEIRO, true);

            }
        }
        $productCall = $callFactory->updateCallAddInvisibleFields($productCall, $invisibleFields);
        $this->entityManager->persist($productCall);

    }

    private function updateIssueTypeProductCall($id, $reference, $product, $classification, $board, $description,
                                                $reportedBy, $callLabels,$devopsId, $menuItemId = null,
                                                $menuVersionId = null, $invisibleFields)
    {
        $connection = $this->entityManager->getConnection();
        $cpCallRepository       = InstanceProvider::getRepository()->getCpCall();
        $operadorRepository     = InstanceProvider::getRepository()->getOperador();
        $clienteRepository      = InstanceProvider::getRepository()->getCliente();
        $callFactory            = InstanceProvider::getFactory()->getCall();
        $pmIntegrationService   = InstanceProvider::getService()->getPmIntegration();
        try {
            $connection->beginTransaction();
            /** @var \Teknisa\Model\Teknisa\Entities\CpCall $productCall */
            $productCall = $cpCallRepository->find($id);
            if($devopsId){
                /** @var \Zeedhi\DevOps\Model\Configurationmanager\Entities\Product $devopsProduct */
                $devopsProduct = InstanceProvider::getRepository()->getProduct()->findOneBy(array('id'=> $devopsId));
                if($devopsProduct && $devopsProduct->getProject()){
                    $board = $devopsProduct->getProject()->getId();
                }
            }
            $productCall = $callFactory->updateCall($productCall, $reference, $product, $classification, $description);
            $this->updateInvisibleFieldsInCall($id, $invisibleFields);
            $reporter = $operadorRepository->findOneBy(array('cdoperador' => $reportedBy));
            $reporterName = $reporter->getNmoperador();

            $cdCliente = $this->environment->getCurrentCdCliente();
            $client = $clienteRepository->findOneBy(array(
                'cdcliente' => $cdCliente,
                'nrorg' => $reporter->getNrorg()
            ));
            $reporterFrom = empty($client) ? null : $client->getNmfantclie();

            $issueId = $productCall->getCallRelationship();


            $issue = $pmIntegrationService->updateIssueByProductCall($issueId, $reference, $product, $classification, $description, $board, $reporterName, $reporterFrom);

            if (empty($issue)) {
                throw new CustomException('ERR_NOT_POSSIBLE_TO_UPDATE_ISSUE_BASED_ON_PRODUCT_CALL');
            }

            /** @var Task $issue */
            $issue = InstanceProvider::getRepository()->getTask()->find($issueId);
            $validateCommentDescription = $this->validateCommentDescription($issue,$menuVersionId,$menuItemId);
            if($menuVersionId){
                $productVersion = InstanceProvider::getRepository()->getProductVersion()->findOneBy(array('id' => $menuVersionId));
                $issue->setProductVersion($productVersion);
            }else{
                $issue->setProductVersion(null);
            }
            $issue->setMenuItem($menuItemId);
            $this->entityManager->persist($issue);

            InstanceProvider::getService()->getCallLabels()->editCallRelationWithLabels($id, $callLabels);

            $this->entityManager->persist($productCall);
            $messageVersion = InstanceProvider::getRepository()->getComments()->getMessageVersion($issue->getId(),Messages::ISSUE_EDIT_VERSION,$product);

            $this->entityManager->flush();
            $connection->commit();

            /** @var Comments $fistCommnet */
            if($validateCommentDescription && $messageVersion && ($menuVersionId || $menuItemId)){
                InstanceProvider::getService()->getPmIntegration()->saveCommentOnIssueByProductCallComment($issue->getId(),
                    $messageVersion,$this->environment->getCurrentUserId(),$this->environment->getCurrentCdCliente());
            }
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    //Retorna dados da tabela CRM_MODELOCASO
    private function retornaDadosModeloCaso($nrorg)
    {
        $nrseqmodelocaso = $this->retornaModeloCasoPararelatione();
        if (!empty($nrseqmodelocaso))
        {
            $sql = "SELECT CDMNEMONICOCASO,     NRSITUACAOABERTURA,     IDGERAFATURCASO,
                           IDGERAEVOLUCAO,      NRTPCONTATOEVOLPADR,    CDMOTIVOEVOLPADR,
                           DSEVOLUCAOPADR,      IDGERAFATUREVOL
                      FROM CRM_MODELOCASO
                     WHERE NRSEQMODELOCASO = ".$nrseqmodelocaso."
                       AND NRORG = ".$nrorg;
            $retorno = $this->entityManager->getConnection()->fetchAll($sql);
            return $retorno[0];
        }
        else
            return null;
    }

    //Verifica se há um modelo de caso inserido na tabela PARARELATIONE
    private function retornaModeloCasoPararelatione()
    {
        $pararelationeRepository = InstanceProvider::getRepository()->getPararelatione();
        return $pararelationeRepository->getNrseqmodelocaso();
    }

    public function findSupportCallCause($referenceProduct)
    {
        $cpCallRepository = InstanceProvider::getRepository()->getCpCall();
        $operador = $this->environment->getCurrentUser();
        $cdOperador = $operador->getCdoperador();
        return $cpCallRepository->findSupportCallCause($cdOperador, $referenceProduct);
    }

    private function getTipoContato($nrParcNegocio, $nrOrg)
    {
        $parcnegocioRepository = InstanceProvider::getRepository()->getParcnegocio();

        $parceiro = $parcnegocioRepository->findOneBy(array(
            'nrparcnegocio' => $nrParcNegocio,
            'nrorg' => $nrOrg
        ));

        $idTpContato = $parceiro->getCdtipoparcprincipal();

        switch ($idTpContato) {
            case 'CLIENTE':
                $tpcontato = 'C';
                break;
            case 'FORNECEDOR':
                $tpcontato = 'F';
                break;
            default:
                $tpcontato = 'P';
        }

        return $tpcontato;
    }

    //Gera o código do caso de acordo com os mnemonicos definidos na tela Modelo de Caso.
    private function geraCodigoCaso($nrorg, $dthrcontato)
    {
        $newCodeService = InstanceProvider::getService()->getNewCode();

        $modelo = $this->retornaDadosModeloCaso($nrorg);
        $contatocd = $newCodeService->getCodeByTableName('CONTATOCD', $nrorg, 1, 10, true );

        if (!empty($modelo)){
            $cdmnemonicocaso = $modelo['CDMNEMONICOCASO'];
            $data = $dthrcontato;
            $data = \Util\DateUtil::getDataDeString($data,\Util\DateUtil::FORMATO_BRASILEIRO_DATAHORA);
            $mes = \Util\DateUtil::getMes($data);
            $ano = \Util\DateUtil::getAno($data);

            if (!empty($cdmnemonicocaso)){
                $cdcaso = str_replace('[SEQ]',$contatocd, $cdmnemonicocaso);
                $cdcaso = str_replace('[MES]',$mes,$cdcaso);
                $cdcaso = str_replace('[ANO]',$ano,$cdcaso);
            } else {
                $cdcaso = $contatocd."_".$mes."/".$ano;
            }
        } else {
            $data = \Util\DateUtil::getDataAtual();
            $mes = \Util\DateUtil::getMes($data);
            $ano = \Util\DateUtil::getAno($data);
            $cdcaso = $contatocd."_".$mes."/".$ano;
        }
        return $cdcaso;
    }

    public function saveNewSupportCall($reference, $description, $product, $cause, $reportedBy, $callLabels,
                                       $callRelationship = null)
    {
        $connection = $this->entityManager->getConnection();
        $operadorRepository = InstanceProvider::getRepository()->getOperador();
        $newCodeService = InstanceProvider::getService()->getNewCode();
        try {
            $connection->beginTransaction();

            if(!$callRelationship){
                /** @var \Teknisa\Model\Geral\Entities\Operador $operador */
                $operador       = $operadorRepository->findOneBy(array('cdoperador' => $reportedBy));
                $cdCliente      = Utilities::getEnvironment()->getCurrentCdCliente();
                $nrOrg          = Utilities::getEnvironment()->getCurrentOrganizationId();
                $dsRefeCont     = $reference;
                $dsAberturaCaso = $description;
                $dtHrContato    = \Util\DateUtil::getDataAtual(true)->format(\Util\DateUtil::FORMATO_BRASILEIRO_DATAHORA);
                $cdCaso         = $this->geraCodigoCaso($nrOrg, $dtHrContato);
                $dtHrContato    = explode(" ", $dtHrContato)[0];
                $dtHrContato    = \DateTime::createFromFormat(\Util\DateUtil::FORMATO_BRASILEIRO, $dtHrContato);
                $cdFilial       = InstanceProvider::getRepository()->getParamGeral()->findOneBy(array())->getCdfilial();
                $cdOperInclusao = $operador->getCdoperador();
                $nrParcInterlocutor = $operador->getNrparcnegocio();
                $cdMotiCont     = $cause;
                $cdSistema      = $product;
                $nrContato      = $newCodeService->getCodeByTableName('CONTATO'.$cdFilial, $nrOrg);
                $nrseqcontato   = $newCodeService->getCodeByTableName('NRSEQCONTATO', $nrOrg);
                $idTpContato    = $this->getTipoContato($operador->getNrparcnegocio(), $nrOrg);
                $nrParcNegocio  = InstanceProvider::getRepository()->getCliente()->findOneBy(array('cdcliente'=>$cdCliente))->getNrParcNegocio();

                $caso = InstanceProvider::getFactory()->getContato()->createContato($cdCaso, $nrOrg, $dsRefeCont, $dsAberturaCaso, $dtHrContato,
                            $nrContato, $cdFilial, $cdCliente, $idTpContato, $cdSistema, $cdMotiCont, $cdOperInclusao,
                            $nrParcInterlocutor, $nrParcNegocio,$nrseqcontato);
                $this->entityManager->persist($caso);
                $callRelationship = $nrseqcontato;
            }

            $supportCall = InstanceProvider::getFactory()->getCall()->createSupportCall($reference, $reportedBy, $description, \Service\Call::TYPE_CASO,
                $callRelationship, $product, $cause);

            $this->entityManager->persist($supportCall);
            $this->entityManager->flush();

            InstanceProvider::getService()->getCallLabels()->createCallRelationWithLabels($supportCall->getNrcall(), $callLabels, $reportedBy);

            $this->entityManager->flush();
            $connection->commit();

            return array("ID" => $supportCall->getNrcall());
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    public function updateSupportCall($id, $reference, $description, $product, $cause, $reportedBy, $callLabels)
    {
        $cpCallRepository   = InstanceProvider::getRepository()->getCpCall();
        $contatoRepository  = InstanceProvider::getRepository()->getContato();
        $callFactory        = InstanceProvider::getFactory()->getCall();
        $contatoFactory     = InstanceProvider::getFactory()->getContato();
        $callLabelsService  = InstanceProvider::getService()->getCallLabels();
        $connection         = $this->entityManager->getConnection();

        try {
            $connection->beginTransaction();

            /** @var \Teknisa\Model\Teknisa\Entities\CpCall $supportCall */
            $supportCall = $cpCallRepository->find($id);

            $supportCall = $callFactory->updateSupportCall($supportCall, $reference, $description, $product, $cause);

            $cdCaso = $supportCall->getCallRelationship();
            $dsRefeCont = $reference;
            $dsAberturaCaso = $description;
            $cdSistema = $product;
            $cdMotiCont = $cause;
            $caso = $contatoRepository->findOneBy(array('nrseqcontato' => $cdCaso));
            $caso = $contatoFactory->updateContato($caso, $dsRefeCont, $dsAberturaCaso, $cdSistema, $cdMotiCont);

            $this->entityManager->persist($caso);

            $callLabelsService->deleteCallLabelRelationBy(array('call' => $id));
            $callLabelsService->createCallRelationWithLabels($id, $callLabels, $reportedBy);

            $this->entityManager->persist($supportCall);

            $this->entityManager->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    private function updateRichClientTypeProductCall($id, $reference, $product, $productType, $productVersion, $useLicense, $classification, $board, $description, $reportedBy, $callLabels, $invisibleFields)
    {
        $connection = $this->entityManager->getConnection();
        try {
            $connection->beginTransaction();
            /** @var \Teknisa\Model\Teknisa\Entities\CpCall $productCall */
            //$productCall = $this->callRepository->find($id);
            //$productCall = $this->callFactory->updateCall($productCall, $reference, $product, $classification, $description);
            $this->updateInvisibleFieldsInCall($id, $invisibleFields);
            InstanceProvider::getService()->getCallLabels()->editCallRelationWithLabels($id, $callLabels);

            //$this->entityManager->persist($productCall);
            $this->entityManager->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    public function updateProductCall($callType, $id,  $reference, $product, $productType, $productVersion,
                                      $useLicense, $classification, $board, $description, $reportedBy,
                                      $callLabels, $devopsId = null, $menuItemId = null, $menuVersionId = null, $invisibleFields = null)
    {
        if($callType == \Service\Call::TYPE_ISSUE) {
            $this->updateIssueTypeProductCall($id, $reference, $product, $classification, $board, $description,
                $reportedBy, $callLabels,$devopsId,$menuItemId,$menuVersionId, $invisibleFields);
        } else if ($callType == \Service\Call::TYPE_RICH_CLIENT) {
            $this->updateRichClientTypeProductCall($id, $reference, $product, $productType, $productVersion, $useLicense, $classification, $board, $description, $reportedBy, $callLabels, $invisibleFields);
        } else {
            throw new CustomException('ERR_NOT_POSSIBLE_TO_SAVE_CALL_DUE_TO_UNKNOWN_TYPE');
        }
    }

    public function saveNewIssueTypeProductCall($callType, $reference, $product, $classification, $board, $description,
                                                 $reportedBy, $callLabels, $callRelationship = null,$devopsId = null,
                                                 $menuItemId = null,$menuVersionId = null, $cdcliente = null, $invisibleFields = null)
    {
        $connection           = $this->entityManager->getConnection();
        $cpCallRepository     = InstanceProvider::getRepository()->getCpCall();
        $pmIntegrationService = InstanceProvider::getService()->getPmIntegration();
        $issue = null;
        try {
            $connection->beginTransaction();

            if($devopsId){
                /** @var \Zeedhi\DevOps\Model\Configurationmanager\Entities\Product $devopsProduct */
                $devopsProduct = InstanceProvider::getRepository()->getProduct()->findOneBy(array('id'=> $devopsId));
                if($devopsProduct && $devopsProduct->getProject()){
                    $board = $devopsProduct->getProject()->getId();
                }
            }

            if($callRelationship){
                $cpCall = $cpCallRepository->findOneBy(array('callRelationship' => $callRelationship));
                if (!empty($cpCall)){
                    throw new CustomException('ERR_CALL_ALREADY_RELATED_ISSUE');
                }
                $resource = $pmIntegrationService->findResourceById($reportedBy);
                if(!empty($resource) && $resource->getExternalCode()){
                    $reportedBy = $resource->getExternalCode();
                } else {
                    $reportedBy = $this->environment->getCurrentUserId();
                }
            }

            $callRelationship = $this->createIssueInPM($reference, $product, $classification,
                $board, $description, $reportedBy, $callRelationship, $cdcliente);

            $issue = $this->setVersionIssue($callRelationship, $menuItemId, $menuVersionId);

            $productCall = $this->callRepository->findOneBy(array('callRelationship'=> $callRelationship));
            if(!$productCall){

                $productCall = InstanceProvider::getFactory()->getCall()->createProductCall($reference,
                    $reportedBy, $description,  $callType, $callRelationship, $product, $classification);
                $this->entityManager->persist($productCall);
            }

            $this->entityManager->flush();

            $this->setFirstCommentIssue($product, $reportedBy, $callLabels, $menuItemId, $menuVersionId,
                $issue, $productCall);

            if($invisibleFields){
                $this->updateInvisibleFieldsInCall($productCall->getNrcall(), $invisibleFields);
            }

            $this->entityManager->flush();
            $connection->commit();

            return array("ID" => $productCall->getNrcall());
        } catch (\Exception $e) {
            $connection->rollBack();
            if (!empty($issue)) {
                $issueId = is_array($issue) ? $issue['ID'] : $issue->getId();
                $pmIntegrationService->removeIssueById($issueId);
            }
            throw $e;
        }
    }

    public function saveNewTicketTypeCall($reference, $cdproduct, $description, $callLabels,
                                                 $callRelationship, $reportedBy,$classification, $invisibleFields = null){
        $connection         = $this->entityManager->getConnection();
        $operadorRepository = InstanceProvider::getRepository()->getOperador();
        $clienteRepository = InstanceProvider::getRepository()->getCliente();
        $callLabelsService  = InstanceProvider::getService()->getCallLabels();
        try {
            $connection->beginTransaction();
            $reporter = $operadorRepository->findOneBy(array('cdoperador' => $reportedBy));
            if(!$reporter){
                $reporter = $this->environment->getCurrentUser();
                $reportedBy = $reporter->getCdoperador();
            }

            $productCall = $this->saveTicketCall($reference, $cdproduct, $description,
                $callRelationship, $reporter,$classification);
            $this->updateInvisibleFieldsInCall($productCall["ID"], $invisibleFields);
            $callLabelsService->createCallRelationWithLabels($productCall->getNrcall(), $callLabels, $reportedBy);

            $connection->commit();
            return array("ID" => $productCall->getNrcall());
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    public function saveNewRichClientTypeProductCall($callType, $reference, $product, $productType, $productVersion,
                                                      $useLicense, $classification, $description, $reportedBy,
                                                      $callLabels, $productRelease, $productPatch, $callRelationship = null,$cdCliente = null)
    {
        $connection         = $this->entityManager->getConnection();
        $cpCallRepository   = InstanceProvider::getRepository()->getCpCall();
        $callLabelsService  = InstanceProvider::getService()->getCallLabels();
        try {
            $connection->beginTransaction();
            if(!$cdCliente){
                $cdCliente = $this->environment->getCurrentCdCliente();
            }
            $callRelationship = $this->saveSolicitacao($reference, $product, $productType, $productVersion, $useLicense,
                $classification, $description, $reportedBy, $productRelease, $productPatch,$cdCliente, $callRelationship);
            $productCall = $cpCallRepository->findOneBy(array("callRelationship" => $callRelationship));
            if (!$productCall) {
                $productCall = InstanceProvider::getFactory()->getCall()->createProductCall($reference, $reportedBy, $description, $callType,
                    $callRelationship, $product, $classification);

                $this->entityManager->persist($productCall);
                $this->entityManager->flush();

                $callLabelsService->createCallRelationWithLabels($productCall->getNrcall(), $callLabels, $reportedBy);

                $connection->commit();

            }

            return array("ID" => $productCall->getNrcall());
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    private function getNrParcNegocioByCdOperador($cdOperador)
    {
        $operadorRepository     = InstanceProvider::getRepository()->getOperador();
        /** @var \Teknisa\Model\Geral\Entities\Operador $operador */
        $operador = $operadorRepository->findOneBy(array('cdoperador' => $cdOperador));
        $nrParcNegocio = $operador->getNrparcnegocio();
        $nrOrg = $operador->getNrorg();

        $query = "SELECT NRPARCNEGOCIO
                  FROM VIEW_INTERPARC
                  WHERE NRTIPORELACIONA = 25 AND NRPARCNEGRELAC  = :NRPARCNEGRELAC
                        AND NRORG = :NRORG AND IDATIVO = 'S'";

        $params = array(
            'NRPARCNEGRELAC' => $nrParcNegocio,
            'NRORG' => $nrOrg
        );

        $return = $this->entityManager->getConnection()->fetchAll($query, $params);
        return !empty($return) ? $return[0]['NRPARCNEGOCIO'] : null;
    }

    public function findResultevol()
    {
        $cpCallRepository = InstanceProvider::getRepository()->getCpCall();
        return $cpCallRepository->findResultevol();
    }

    public function findSupportCallProducts()
    {
        $cpCallRepository = InstanceProvider::getRepository()->getCpCall();
        $operador   = $this->environment->getCurrentUser();
        $cdOperador = $operador->getCdoperador();
        $nrOrg      = $operador->getNrorg();
        return $cpCallRepository->findSupportCallReferenceProducts($cdOperador, $nrOrg);
    }

    public function deleteSupportCall($supportCallID, $associatedCasoID)
    {
        $cpCallRepository   = InstanceProvider::getRepository()->getCpCall();
        $contatoRepository  = InstanceProvider::getRepository()->getContato();
        $callLabelsService  = InstanceProvider::getService()->getCallLabels();
        $connection = $this->entityManager->getConnection();
        try {
            $connection->beginTransaction();

            $supportCall = $cpCallRepository->find($supportCallID);
            $caso = $contatoRepository->findOneBy(array('cdcaso' => $associatedCasoID));

            $this->entityManager->remove($caso);
            $this->entityManager->remove($supportCall);

            $callLabelsService->deleteCallLabelRelationBy(array('call' => $supportCallID));

            $this->entityManager->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    public function saveNewRichClientTypeProductCallBySoliserv($soliservId)
    {
        $soliserv =  $this->findSoliservById($soliservId);
        $descriptionInTextPlain = Utilities::rtf2textPlain($soliserv['DESCRIPTION']);
        $productCall =  $this->saveNewRichClientTypeProductCall('S', $soliserv['REFERENCE'],
                        $soliserv['PRODUCT'], $soliserv['PRODUCT_TYPE'], $soliserv['PRODUCT_VERSION'],
                        $soliserv['USE_LICENSE'], $soliserv['CLASSIFICATION'], $descriptionInTextPlain, $soliserv['REPORTER_BY'],
                        array(), $soliserv['PRODUCT_RELEASE'], $soliserv['PRODUCT_PATCH'], $soliserv['ID']);

        return $productCall;
    }

    public function saveNewProductCall($callType, $reference, $product, $productType, $productVersion, $useLicense,
                                       $classification, $board, $description, $reportedBy, $callLabels, $productRelease,
                                       $productPatch, $callRelationship = null,$devopsId = null, $menuItemId = null,
                                       $menuVersionId = null,$cdcliente = null, $invisibleFields = null)
    {
        if($callType == \Service\Call::TYPE_ISSUE) {
            $newProductCall = $this->saveNewIssueTypeProductCall($callType, $reference, $product, $classification, $board, $description,
                $reportedBy, $callLabels, $callRelationship,$devopsId,$menuItemId,$menuVersionId,$cdcliente, $invisibleFields);
        } else if ($callType == \Service\Call::TYPE_RICH_CLIENT) {
            $newProductCall = $this->saveNewRichClientTypeProductCall($callType, $reference, $product, $productType, $productVersion,
                $useLicense, $classification, $description, $reportedBy, $callLabels, $productRelease, $productPatch,
                $callRelationship,$cdcliente);
        } else if ($callType == \Service\Call::TYPE_TICKET) {
            $newProductCall =  $this->saveNewTicketTypeCall($reference, $product, $description, $callLabels,
                $callRelationship, $reportedBy,$classification, $invisibleFields);
        } else {
            throw new CustomException('ERR_NOT_POSSIBLE_TO_SAVE_CALL_DUE_TO_UNKNOWN_TYPE');
        }
        $cpCallRepository = InstanceProvider::getRepository()->getCpCall();

        /** @var Entities\CpCall $cpCall */
        $cpCall = $cpCallRepository->find($newProductCall['ID']);

        $newProductCall['CALL_RELATIONSHIP'] = $cpCall->getCallRelationship();

        return $newProductCall;
    }

    public function findProductCallProductTypes($filterString)
    {
        $query = "
            SELECT *
              FROM( SELECT P.CDPRODUTO as ID, P.NMPRODUTO AS LABEL
                      FROM PRODUTO P, VERSAOPROD V
                     WHERE (P.CDNVPRODUTO = '3')
                       AND (NOT P.CDARVPROD LIKE 'P4%')
                       AND (
                               (P.NMPRODUTO LIKE '%Sistemas%Vers%')
                                     OR (UPPER(P.NMPRODUTO) LIKE '%SITE%JAVA%')
                                     OR (UPPER(P.NMPRODUTO) LIKE 'PROD%SETFORMS%VERS%')
                                     OR (P.CDPRODUTO IN ('0000001386', '0000004180'))
                                    AND NOT P.NMPRODUTO LIKE '%Componente%'
                           )
                       AND P.CDPRODUTO = V.CDPRODGRUP(+)
                       AND NVL(V.IDVERSAOATIV, 'N') = 'S'
                    ORDER BY 2 DESC
                  )
                WHERE 1=1
            $filterString
        ";
        return $this->entityManager->getConnection()->fetchAll($query);
    }

    public function findProductCallUseLicense($filterString,$cdCliente = null)
    {

        if(!$cdCliente){
            $cdCliente = $this->environment->getCurrentCdCliente();
        }

        $query = "
            SELECT *
              FROM (SELECT NRLICENCAUSO as ID, NRLICENCAUSO || ' - ' || NMFANTLUSO as LABEL
                      FROM LICENCAUSO
                     WHERE ((NRLICENCAUSO = :P_NRLICENCAUSO) OR ('T' = :P_TNRLICENCAUSO))
                       AND (CDCLIENTE = :P_CDCLIENTE)
                  ORDER BY NRLICENCAUSO)
             WHERE 1 = 1
             $filterString
        ";

        $params = array(
            'P_TNRLICENCAUSO' => "T",
            'P_NRLICENCAUSO' => "",
            'P_CDCLIENTE' => $cdCliente,
        );

        return $this->entityManager->getConnection()->fetchAll($query, $params);
    }

    public function findProductCallTypes($isNew)
    {
        $thinClient = array(
            'ID' => \Service\Call::TYPE_ISSUE,
            'ID_FILTER' => \Service\Call::TYPE_ISSUE,
            'NMMOTICONT' => null,
            'LABEL' => $this->getCallTypeDescription(\Service\Call::TYPE_ISSUE)
        );

        $richClient = array(
            'ID' => \Service\Call::TYPE_RICH_CLIENT,
            'ID_FILTER' => \Service\Call::TYPE_RICH_CLIENT,
            'NMMOTICONT' => null,
            'LABEL' => $this->getCallTypeDescription(\Service\Call::TYPE_RICH_CLIENT)
        );


        $ticket =  InstanceProvider::getRepository()->getCpCall()->getCallTypeTicket(self::TYPE_TICKET);

        if($isNew && !Utilities::currentUserIsTeknisa() && Utilities::getActiveTicket()){
            $result = $ticket;
        }else{
            $result = array($thinClient,$richClient);
            $result = array_merge($result,$ticket);
        }
        return $result;
    }

    public function validateCdCliente($cdcliente){
        $operadorService = \Util\Instance\Service::getOperador();
        $condition = array(
            array(
                "columnName" => "CDCLIENTE",
                "operator" => "IN",
                "value" => $cdcliente,
            )
        );
        $dataSet = $operadorService->findAllAvailableCliente($condition);
        return !empty($dataSet);

    }


    public function findCalls($mainType, $conditions, $orderBy, $page, $pageSize, $clientes = null,$groupByLabel = 'N')
    {
        $limit = $page * $pageSize;
        $offset = ($page - 1) * $pageSize;
        $cpCallRepository = InstanceProvider::getRepository()->getCpCall();
        $userId           = $this->environment->getCurrentUserId();
        $situationFilters = $this->getAllSituationDividedByType();



        if(!$clientes) {
            if ($this->environment->currentUserIsTeknisa()) {
                $clientes = "1 = 1";
            } else {
                $operadorRepository = InstanceProvider::getRepository()->getOperador();
                $clientes = $operadorRepository->findAvailableCliente($this->environment->getCurrentUserId(), 0);
                $clientes = DevOpsUtilities::getArrayFromArrayAssociative($clientes, "CDCLIENTE");
            }
        } else
            if(!is_array($clientes)) {
                array($clientes);
            }


        $clientePmFilter = $clientes == "1 = 1" ? $clientes : DevOpsUtilities::fixBugWhereINQuery($clientes, "SRE.EXTERNAL_CODE", false, true);
        $clienteIntranetFilter = $clientes == "1 = 1" ? $clientes : DevOpsUtilities::fixBugWhereINQuery($clientes, "SER.CDCLIENTE", false, true);
        $clienteCRMFilter = $clientes == "1 = 1" ? $clientes : DevOpsUtilities::fixBugWhereINQuery($clientes, "CLI.CDCLIENTE", false, true);

        if($orderBy) {
            $constructOrderBy = null;
            foreach ($orderBy as $i => $order) {
                $this->orderReverse = $order == 'ASC';
                $orderByName        = $i;
                $this->orderBy      = $orderByName;
                if($constructOrderBy){
                    $constructOrderBy .= ", CALL." . $orderByName ." ". $order;

                }else{
                    $constructOrderBy  = "CALL." . $orderByName ." ". $order;
                }
            }
            $orderBy = $constructOrderBy;
        } else {
            $orderBy = "CALL.UPDATED_AT DESC";
        }
        $orderBy .= " NULLS LAST, TEKNISA_ID DESC";
         $customQuery = $this->getQueryAllCall($conditions, $mainType, $situationFilters, $clientePmFilter,
             $clienteIntranetFilter, $clienteCRMFilter);
        $result = array();
        if($customQuery){
            $result = $cpCallRepository->findAllCalls($customQuery, $situationFilters,  $userId,  $offset, $limit, $orderBy,$groupByLabel);

            foreach($result as &$retorno){
                for($i=1; $i<6;$i++){
                    $retorno["DATE_" .$i] = $retorno["DATE_". $i] ? $retorno["DATE_". $i] : "";
                }
            }
        }
        return $result;
    }

    public function updateSolicitationEndDate($updateParams){

        $cpCallRepository   = InstanceProvider::getRepository()->getCpCall();
        $cpCallRepository->updateSolicitationEndDate($updateParams);

    }

    public function updateTaskEndDate($taskId, $endDate){

        $taskRepository = InstanceProvider::getRepository()->getTask();

        $issue = $taskRepository->findOneBy(array('id' => $taskId));
        $endDate = \DateTime::createFromFormat('d/m/Y', $endDate);

        $issue->setEndDate($endDate);

        $this->entityManager->persist($issue);
        $this->entityManager->flush();

    }

    public function saveDefaultPreferences($view, $filter, $layout) {
        $now       = new \DateTime();
        $userId    = $this->environment->getCurrentUserId();
        $cdcliente = Utilities::getEnvironment()->getCurrentCdCliente();
        $nrorg     = Utilities::getEnvironment()->getCurrentOrganizationId();
        $cpCustomerPreferences = InstanceProvider::getRepository()->getCpCustomerPreferences()->findOneBy(array('cdcliente' => $cdcliente));
        $entity = $this->cpCustomerPreferencesFactory->saveDefaultPreferences($cpCustomerPreferences, $view, $layout, $filter, $cdcliente, $nrorg, $now, $userId);
        $this->entityManager->persist($entity);
        $this->entityManager->flush();
    }

    public function findDefaultPreferences() {
        $cdcliente = Utilities::getEnvironment()->getCurrentCdCliente();
        $cpCustomerPreferencesRepository = InstanceProvider::getRepository()->getCpCustomerPreferences()->findOneBy(array('cdcliente' => $cdcliente));
        $preference = [];
        if($cpCustomerPreferencesRepository) {
            $preference["view"]   = $cpCustomerPreferencesRepository->getViewId();
            $preference["filter"] = $cpCustomerPreferencesRepository->getFilterId();
            $preference["layout"] = $cpCustomerPreferencesRepository->getLayoutId();
        }
        return $preference;
    }

    public function findPreferencesPerId($id, $type) {
        $preference = InstanceProvider::getRepository()->getCpCustomerPreferences()->findPreferencesPerId($id, $type);
        return $preference;
    }
    
    public function slaDetails($cdfilial, $callRelationship) {
        
        $results = InstanceProvider::getRepository()->getCpCall()->findSLADetails($cdfilial, $callRelationship);
        foreach($results as &$result){
            $nrsaldo = $this->calcularSaldo($cdfilial, $result['DTFIMPREVSLA'], $result['DTFINAL']);
            $qtDiasReal = $this->calcularSaldo($cdfilial, $result['DTFINAL'], $result['DTINIPREVSLA']);
            
            $result['QTDIASREAL'] = $qtDiasReal;
            if($result['IDRESPFASE'] == 'T'){
                $result['DBTOTALTEK'] = $nrsaldo;
            }else{
                $result['DBTOTALCLI'] = $nrsaldo;
            }
            
        }
        return $results;
    }
    
    public function verifySlaVisibility($cdcliente, $callRelationship) {
        
        $results = InstanceProvider::getRepository()->getCpCall()->verifySlaVisibility($cdcliente, $callRelationship)[0]['CONTADOR'];
        return $results;
    }
    
    public function calcularSaldo($cdfilial, $dataPrev, $dataAtual) {
        $dtAux = $dataPrev;
        $saldoAux =  $this->calcularDatas($dataPrev, $dataAtual);
        $result =  $saldoAux;
        $diasNaoUteis = InstanceProvider::getRepository()->getCpCall()->findDiasNaoUteis($cdfilial, $dataPrev, $dataAtual)[0]['DIAS'];
   
        if($result > 0){
            $result = $result - $diasNaoUteis;
        }else if ($result < 0){
             $result = $result + $diasNaoUteis;
        }
        return $result;
        
        
    }
    
    public function calcularDatas($dataPrev, $dataAtual) { 
    $date = str_replace('/', '-', $dataPrev);
    $dateP = date('Y-m-d', strtotime($date));
    
    $date = str_replace('/', '-', $dataAtual);
    $dateA = date('Y-m-d', strtotime($date));
    $diff = strtotime($dateP) - strtotime($dateA); 
    $result = abs(round($diff / 86400)); 
   if($dateA > $dateP){
        $result = $result * -1;
      }
    return $result;
} 
    
}
